@@ -179,6 +179,8 @@ CDECL const char *strerror(int);
 # define textdomain(Domain)
 # define _(Text) Text
 # define gettext(z) z
+# define dgettext(d,z) z
+# define dcgettext(d,z,c) z
 #endif
 #define N_(Text) Text
 
@@ -248,4 +250,17 @@ CDECL void SOCKSinit(const char *);
 # define __builtin_expect(expr, val) (expr)
 #endif
 
+#if defined(__cplusplus) && !defined(HAVE_CXX_BOOL)
+typedef unsigned _bool;
+#define bool   _bool
+#define false  0U
+#define true   1U
+#endif
+
+#ifdef __GNUC__
+# define PRINTF_LIKE(n,m) __attribute__((format(printf,n,m)))
+#else
+# define PRINTF_LIKE(n,m)
+#endif
+
 #endif /* CONFIG_H */
