@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: FileAccess.h,v 1.60 2001/08/27 13:56:49 lav Exp $ */
+/* $Id: FileAccess.h,v 1.65 2001/10/25 10:46:33 lav Exp $ */
 
 #ifndef FILEACCESS_H
 #define FILEACCESS_H
@@ -134,6 +134,12 @@ protected:
 	 xfree(real_cwd);
 	 real_cwd=xstrdup(c);
       }
+   void set_home(const char *h)
+      {
+	 xfree(home);
+	 home=xstrdup(h);
+	 ExpandTildeInCWD();
+      }
 
    char *url;
 
@@ -143,6 +149,10 @@ protected:
    char *closure;
    const char *res_prefix;
    ResValue Query(const char *name,const char *closure=0);
+   bool QueryBool(const char *name,const char *closure=0)
+      {
+	 return Query(name,closure).to_bool();
+      }
 
    int chmod_mode;
    bool ascii;
@@ -185,11 +195,12 @@ public:
 
    virtual void Connect(const char *h,const char *p);
    virtual void ConnectVerify();
-
    virtual void AnonymousLogin();
-
    virtual void Login(const char *u,const char *p);
 
+   // reset location-related state on Connect/Login/AnonymousLogin
+   virtual void ResetLocationData();
+
    virtual void Open(const char *file,int mode,off_t pos=0);
    void SetFileURL(const char *u) { xfree(file_url); file_url=xstrdup(u); }
    void SetSize(off_t s) { entity_size=s; }
@@ -361,6 +372,7 @@ protected:
    bool	 files_only;
    bool	 match_period;
    bool	 inhibit_tilde;
+   bool	 casefold;
    void	 add(const FileInfo *info);
    void	 add_force(const FileInfo *info);
    virtual ~Glob();
@@ -372,6 +384,7 @@ public:
    void FilesOnly() { files_only=true; }
    void NoMatchPeriod() { match_period=false; }
    void NoInhibitTilde() { inhibit_tilde=false; }
+   void CaseFold() { casefold=true; }
 
    static bool HasWildcards(const char *);
    static void UnquoteWildcards(char *);
@@ -421,6 +434,7 @@ public:
 
    virtual void SetExclude(const char *p,regex_t *x,regex_t *i);
 
+   // caller has to delete the resulting FileSet itself.
    FileSet *GetResult()
       {
 	 FileSet *tmp=result;
