@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: FileCopy.h,v 1.39 2001/02/03 11:37:50 lav Exp $ */
+/* $Id: FileCopy.h,v 1.42 2001/11/06 13:15:32 lav Exp $ */
 
 /*
   classes defined here:
@@ -59,6 +59,8 @@ protected:
    bool ascii;
    bool use_cache;
 
+   bool write_allowed;
+
 public:
    enum direction { GET, PUT };
 
@@ -119,6 +121,10 @@ public:
    virtual FA *GetSession() { return 0; } // for fxp.
    virtual void Fg() {}
    virtual void Bg() {}
+
+   void AllowWrite() { write_allowed=true; }
+   bool WriteAllowed() { return write_allowed; }
+   bool WritePending() { return mode==PUT && Size()>0; }
 };
 
 class FileCopy : public SMTask
@@ -151,10 +157,8 @@ private:
    int put_buf;
    off_t put_eof_pos;
 
-   time_t start_time;
-   int start_time_ms;
-   time_t end_time;
-   int end_time_ms;
+   Time start_time;
+   Time end_time;
 
    bool fail_if_cannot_seek;
    bool remove_source_later;
@@ -171,7 +175,6 @@ protected:
    void RateReset()
       {
 	 start_time=now;
-	 start_time_ms=now_ms;
 	 rate->Reset();
 	 rate_for_eta->Reset();
       }
@@ -196,8 +199,7 @@ public:
    pid_t GetProcGroup();
    void Kill(int sig);
    off_t GetBytesCount() { return bytes_count; }
-   time_t GetTimeSpent();
-   int GetTimeSpentMilli();
+   double GetTimeSpent();
 
    void SetDate(time_t t) { get->SetDate(t); }
    void SetSize(off_t  s) { get->SetSize(s); }
@@ -227,6 +229,10 @@ public:
 
    static FileCopy *New(FileCopyPeer *src,FileCopyPeer *dst,bool cont);
    static FileCopy *(*fxp_create)(FileCopyPeer *src,FileCopyPeer *dst,bool cont);
+
+   void AllowWrite() { put->AllowWrite(); }
+   bool WriteAllowed() { return !put || put->WriteAllowed(); }
+   bool WritePending() { return put && put->WritePending(); }
 };
 
 class FileCopyPeerFA : public FileCopyPeer
@@ -343,9 +349,10 @@ public:
    FileCopyPeerDirList(FA *s,ArgV *v); // consumes s and v.
 
    int Do();
-   void NoCache() { if(dl) dl->UseCache(false); }
+   void NoCache() { use_cache=false; if(dl) dl->UseCache(false); }
    void Fg() { session->SetPriority(1); }
    void Bg() { session->SetPriority(0); }
+   const char *GetStatus() { return session->CurrentStatus(); }
 };
 
 #endif
