@@ -34,6 +34,7 @@
 #include "LsCache.h"
 #include "misc.h"
 #include "log.h"
+#include "ArgV.h"
 
 #define super NetAccess
 
@@ -90,7 +91,7 @@ int Fish::Do()
    // check if idle time exceeded
    if(mode==CLOSED && send_buf && idle>0)
    {
-      if(now-idle_start>=idle)
+      if(now >= idle_start+idle)
       {
 	 DebugPrint("---- ",_("Closing idle connection"),1);
 	 Disconnect();
@@ -151,69 +152,58 @@ int Fish::Do()
       if(!NextTry())
 	 return MOVED;
 
-      if(pipe(pipe_in)==-1)
-      {
-	 if(errno==EMFILE || errno==ENFILE)
-	 {
-	    TimeoutS(1);
-	    return m;
-	 }
-	 SetError(SEE_ERRNO,"pipe()");
-	 return MOVED;
-      }
-      NonBlock(pipe_in[0]);
-      CloseOnExec(pipe_in[0]);
-      CloseOnExec(pipe_in[1]);
+      const char *shell=Query("shell",hostname);
+      char *init=alloca_strdup2("echo FISH:;",strlen(shell));
+      strcat(init,shell);
 
-      const char *init="\"echo FISH:;/bin/bash\"";
-      char *cmd=string_alloca(xstrlen(hostname)*2+xstrlen(portname)*2
-			      +xstrlen(user)*2+128);
-      strcpy(cmd,"ssh");
+      ArgV *cmd=new ArgV("ssh");
+      cmd->Add("-x");	// don't forward X11
+      cmd->Add("-a");	// don't forward AuthAgent.
       if(user)
       {
-	 strcat(cmd," -l ");
-	 strcat(cmd,shell_encode(user));
+	 cmd->Add("-l");
+	 cmd->Add(user);
       }
       if(portname)
       {
-	 strcat(cmd," -p ");
-	 strcat(cmd,shell_encode(portname));
+	 cmd->Add("-p");
+	 cmd->Add(portname);
       }
-      strcat(cmd," ");
-      strcat(cmd,shell_encode(hostname));
-      strcat(cmd," ");
-      strcat(cmd,init);
-      Log::global->Format(9,"---- %s (%s)\n",_("Running ssh..."),cmd);
-      filter_out=new OutputFilter(cmd,pipe_in[1]);
-      filter_out->StderrToStdout();
+      cmd->Add(hostname);
+      cmd->Add(init);
+      char *cmd_str=cmd->Combine(0);
+      Log::global->Format(9,"---- %s (%s)\n",_("Running ssh..."),cmd_str);
+      xfree(cmd_str);
+      ssh=new PtyShell(cmd);
       state=CONNECTING;
       event_time=now;
       m=MOVED;
    }
    case CONNECTING:
-      fd=filter_out->getfd();
+      fd=ssh->getfd();
       if(fd==-1)
       {
-	 if(filter_out->error())
+	 if(ssh->error())
 	 {
-	    SetError(FATAL,filter_out->error_text);
+	    SetError(FATAL,ssh->error_text);
 	    return MOVED;
 	 }
 	 TimeoutS(1);
 	 return m;
       }
-      filter_out->Kill(SIGCONT);
-      send_buf=new IOBufferFDStream(filter_out,IOBuffer::PUT);
-      filter_out=0;
-      recv_buf=new IOBufferFDStream(new FDStream(pipe_in[0],"pipe in"),IOBuffer::GET);
-      close(pipe_in[1]);
-      pipe_in[1]=-1;
-      state=CONNECTED;
+      ssh->Kill(SIGCONT);
+      send_buf=new IOBufferFDStream(ssh,IOBuffer::PUT);
+      ssh=0;
+      recv_buf=new IOBufferFDStream(new FDStream(fd,"pseudo-tty"),IOBuffer::GET);
       set_real_cwd("~");
+      state=CONNECTING_1;
       m=MOVED;
 
+   case CONNECTING_1:
+      if(!received_greeting)
+	 return m;
       Send("#FISH\n"
-	   "echo;start_fish_server;"
+	   "exec 2>&1;echo;start_fish_server;"
 	   "TZ=GMT;export TZ;LC_ALL=C;export LC_ALL;"
 	   "echo '### 200'\n");
       PushExpect(EXPECT_FISH);
@@ -226,6 +216,9 @@ int Fish::Do()
 	      "pwd; echo '### 200'\n");
 	 PushExpect(EXPECT_PWD);
       }
+      state=CONNECTED;
+      m=MOVED;
+
    case CONNECTED:
       if(mode==CLOSED)
 	 return m;
@@ -315,8 +308,6 @@ void Fish::MoveConnectionHere(Fish *o)
 {
    send_buf=o->send_buf; o->send_buf=0;
    recv_buf=o->recv_buf; o->recv_buf=0;
-   pipe_in[0]=o->pipe_in[0]; o->pipe_in[0]=-1;
-   pipe_in[1]=o->pipe_in[1]; o->pipe_in[1]=-1;
    rate_limit=o->rate_limit; o->rate_limit=0;
    path_queue=o->path_queue; o->path_queue=0;
    path_queue_len=o->path_queue_len; o->path_queue_len=0;
@@ -337,20 +328,13 @@ void Fish::Disconnect()
       DebugPrint("---- ",_("Disconnecting"),9);
    Delete(send_buf); send_buf=0;
    Delete(recv_buf); recv_buf=0;
-   if(pipe_in[0]!=-1)
-      close(pipe_in[0]);
-   pipe_in[0]=-1;
-   if(pipe_in[1]!=-1)
-      close(pipe_in[1]);
-   pipe_in[1]=-1;
-   if(filter_out)
-      delete filter_out;
-   filter_out=0;
+   delete ssh; ssh=0;
    EmptyRespQueue();
    EmptyPathQueue();
    state=DISCONNECTED;
    if(mode==STORE)
       SetError(STORE_FAILED,0);
+   received_greeting=false;
 }
 
 void Fish::EmptyPathQueue()
@@ -366,8 +350,7 @@ void Fish::Init()
    send_buf=0;
    recv_buf=0;
    recv_buf_suspended=false;
-   pipe_in[0]=pipe_in[1]=-1;
-   filter_out=0;
+   ssh=0;
    max_send=0;
    line=0;
    message=0;
@@ -377,6 +360,7 @@ void Fish::Init()
    eof=false;
    path_queue=0;
    path_queue_len=0;
+   received_greeting=false;
 }
 
 Fish::Fish()
@@ -416,6 +400,7 @@ void Fish::Close()
       break;
    case(FILE_RECV):
    case(CONNECTING):
+   case(CONNECTING_1):
       Disconnect();
    }
 //    if(!RespQueueIsEmpty())
@@ -430,14 +415,15 @@ void Fish::Close()
 void Fish::Send(const char *format,...)
 {
    va_list va;
-   va_start(va,format);
    char *str;
 
    static int max_send=256;
    for(;;)
    {
+      va_start(va,format);
       str=string_alloca(max_send);
       int res=vsnprintf(str,max_send,format,va);
+      va_end(va);
       if(res>=0 && res<max_send)
       {
 	 if(res<max_send/16)
@@ -449,7 +435,6 @@ void Fish::Send(const char *format,...)
 
    DebugPrint("---> ",str,5);
    send_buf->Put(str);
-   va_end(va);
 }
 
 const char *Fish::shell_encode(const char *string)
@@ -649,6 +634,31 @@ int Fish::HandleReplies()
    const char *eol=(const char*)memchr(b,'\n',s);
    if(!eol)
    {
+      if(state==CONNECTING_1)
+      {
+	 const char *p="password: ";
+	 const char *y="(yes/no)? ";
+	 int p_len=strlen(p);
+	 int y_len=strlen(y);
+	 if(s>=p_len && !strncasecmp(b+s-p_len,p,p_len))
+	 {
+	    if(!pass)
+	    {
+	       SetError(LOGIN_FAILED,"Password required");
+	       return MOVED;
+	    }
+	    recv_buf->Put("XXXX");
+	    send_buf->Put(pass);
+	    send_buf->Put("\n");
+	    return m;
+	 }
+	 if(s>=y_len && !strncasecmp(b+s-y_len,y,y_len))
+	 {
+	    recv_buf->Put("yes\n");
+	    send_buf->Put("yes\n");
+	    return m;
+	 }
+      }
       if(recv_buf->Eof() || recv_buf->Error())
 	 goto hup;
       return m;
@@ -667,6 +677,11 @@ int Fish::HandleReplies()
    DebugPrint("<--- ",line,ReplyLogPriority(code));
    if(code==-1)
    {
+      if(!received_greeting && !strcmp(line,"FISH:"))
+      {
+	 received_greeting=true;
+	 return m;
+      }
       if(message==0)
 	 message=xstrdup(line);
       else
@@ -695,14 +710,12 @@ int Fish::HandleReplies()
    {
    case EXPECT_FISH:
    case EXPECT_VER:
-      // nothing (yet).
+      /* nothing yet */
       break;;
    case EXPECT_PWD:
       if(!message)
 	 break;
-      xfree(home);
-      home=message;
-      message=0;
+      set_home(message);
       break;
    case EXPECT_CWD:
       p=PopDirectory();
@@ -1062,12 +1075,13 @@ const char *Fish::CurrentStatus()
    switch(state)
    {
    case DISCONNECTED:
+      if(!ReconnectAllowed())
+	 return DelayingMessage();
       return _("Not connected");
    case CONNECTING:
+   case CONNECTING_1:
       return _("Connecting...");
    case CONNECTED:
-      if(!RespQueueIsEmpty() && RespQueue[RQ_head]==EXPECT_FISH)
-	 return _("Connecting...");
       return _("Connected");
    case WAITING:
       return _("Waiting for response...");
@@ -1314,7 +1328,7 @@ static FileInfo *ls_to_FileInfo(char *line)
       {
 	 sprintf(month_name,"%02d",month+1);
 	 if(year==-1)
-	    year=guess_year(month,day);
+	    year=guess_year(month,day,hour,minute,SMTask::now);
       }
 
       FileInfo *fi=new FileInfo;
@@ -1341,6 +1355,7 @@ static FileInfo *ls_to_FileInfo(char *line)
       }
 
       fi->SetSize(size);
+      fi->SetMode(perms_code);
 
       return fi;
    }
