@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 2000 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 2000-2001 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,13 +18,13 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: Fish.h,v 1.12 2001/06/29 14:57:07 lav Exp $ */
+/* $Id: Fish.h,v 1.15 2001/11/08 09:25:48 lav Exp $ */
 
 #ifndef FISH_H
 #define FISH_H
 
 #include "NetAccess.h"
-#include "StatusLine.h"
+#include "PtyShell.h"
 
 class Fish : public NetAccess
 {
@@ -32,6 +32,7 @@ class Fish : public NetAccess
    {
       DISCONNECTED,
       CONNECTING,
+      CONNECTING_1,
       CONNECTED,
       FILE_RECV,
       FILE_SEND,
@@ -40,6 +41,7 @@ class Fish : public NetAccess
    };
 
    state_t state;
+   bool received_greeting;
 
    void Init();
 
@@ -52,8 +54,7 @@ class Fish : public NetAccess
    IOBuffer *recv_buf;
    bool recv_buf_suspended;
 
-   OutputFilter *filter_out;  // used in connecting
-   int pipe_in[2];	      // used in connecting
+   PtyShell *ssh;
 
    void Disconnect();
    int IsConnected()
