@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: Http.cc,v 1.131 2001/07/25 13:21:45 lav Exp $ */
+/* $Id: Http.cc,v 1.137 2001/11/06 13:15:32 lav Exp $ */
 
 #include <config.h>
 #include <stdio.h>
@@ -249,14 +249,15 @@ void Http::Close()
 void Http::Send(const char *format,...)
 {
    va_list va;
-   va_start(va,format);
    char *str;
 
    static int max_send=256;
    for(;;)
    {
+      va_start(va,format);
       str=string_alloca(max_send);
       int res=vsnprintf(str,max_send,format,va);
+      va_end(va);
       if(res>=0 && res<max_send)
       {
 	 if(res<max_send/16)
@@ -268,7 +269,6 @@ void Http::Send(const char *format,...)
 
    DebugPrint("---> ",str,5);
    send_buf->Put(str);
-   va_end(va);
 }
 
 void Http::SendMethod(const char *method,const char *efile)
@@ -295,7 +295,7 @@ void Http::SendMethod(const char *method,const char *efile)
    if(hftp && mode!=LONG_LIST && mode!=CHANGE_DIR && mode!=MAKE_DIR
    && mode!=REMOVE && mode!=REMOVE_DIR
    && (strlen(efile)<7 || strncmp(efile+strlen(efile)-7,";type=",6))
-   && (bool)Query("use-type",hostname))
+   && QueryBool("use-type",hostname))
    {
       char *pfile=alloca_strdup2(efile,7);
       sprintf(pfile,"%s;type=%c",efile,ascii?'a':'i');
@@ -359,7 +359,7 @@ void Http::SendAuth()
 {
    if(proxy && proxy_user && proxy_pass)
       SendBasicAuth("Proxy-Authorization",proxy_user,proxy_pass);
-   if(user && pass && !(hftp && !(bool)Query("use-authorization",proxy)))
+   if(user && pass && !(hftp && !QueryBool("use-authorization",proxy)))
       SendBasicAuth("Authorization",user,pass);
 }
 
@@ -410,7 +410,7 @@ void Http::SendRequest(const char *connection,const char *f)
 	 {
 	    strcpy(pfile,"ftp://");
 	    url::encode_string(user,pfile+strlen(pfile),URL_USER_UNSAFE);
-	    if(!(bool)Query("use-authorization",proxy))
+	    if(!QueryBool("use-authorization",proxy))
 	    {
 	       strcat(pfile,":");
 	       url::encode_string(pass,pfile+strlen(pfile),URL_PASS_UNSAFE);
@@ -693,7 +693,7 @@ void Http::HandleHeaderLine(const char *name,const char *value)
       return;
    }
    if(!strcasecmp(name,"Set-Cookie")
-   && !hftp && (bool)Query("set-cookies",hostname))
+   && !hftp && QueryBool("set-cookies",hostname))
       SetCookie(value);
 }
 
@@ -764,7 +764,7 @@ int Http::Do()
    // check if idle time exceeded
    if(mode==CLOSED && sock!=-1 && idle>0)
    {
-      if(now-idle_start>=idle)
+      if(now >= idle_start+idle)
       {
 	 DebugPrint("---- ",_("Closing idle connection"),1);
 	 Disconnect();
@@ -1619,7 +1619,7 @@ void Http::Reconfig(const char *name)
 
    super::Reconfig(name);
 
-   no_cache = !(bool)Query("cache",c);
+   no_cache = !QueryBool("cache",c);
    if(!hftp && NoProxy(hostname))
       SetProxy(0);
    else
@@ -1668,9 +1668,9 @@ bool Http::SameLocationAs(FileAccess *fa)
    return true;
 }
 
-void Http::Connect(const char *new_host,const char *new_port)
+void Http::ResetLocationData()
 {
-   super::Connect(new_host,new_port);
+   super::ResetLocationData();
    Reconfig();
    state=DISCONNECTED;
 }
@@ -1702,10 +1702,12 @@ bool Http::CookieClosureMatch(const char *closure_c,
       char *slash=strchr(scan,';');
       if(!slash)
 	 break;
-      *slash=0;
-      if(!strncmp(slash+1,"path=",5))
-	 path=slash+1+5;
-      else if(!strncmp(slash+1,"secure",6) && (slash[7]==';' || slash[7]==0))
+      *slash++=0;
+      while(*slash && *slash==' ')
+	 slash++;
+      if(!strncmp(slash,"path=",5))
+	 path=slash+5;
+      else if(!strncmp(slash,"secure",6) && (slash[6]==';' || slash[6]==0))
       {
 	 if(!https)
 	    return false;
@@ -1947,7 +1949,7 @@ void HFtp::Login(const char *u,const char *p)
 void HFtp::Reconfig(const char *name)
 {
    super::Reconfig(name);
-   use_head=Query("use-head");
+   use_head=QueryBool("use-head");
 }
 
 void Http::Cleanup()
