@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: ResMgr.h,v 1.24 2001/08/22 07:19:07 lav Exp $ */
+/* $Id: ResMgr.h,v 1.28 2001/11/09 16:00:29 lav Exp $ */
 
 #ifndef RESMGR_H
 #define RESMGR_H
@@ -27,6 +27,7 @@
 #include <sys/types.h>
 #include <time.h>
 #include "xmalloc.h"
+#include "TimeDate.h"
 
 typedef const char *ResValValid(char **value);
 typedef const char *ResClValid(char **closure);
@@ -74,6 +75,7 @@ public:
    static const char *QueryNext(const char *name,const char **closure,Resource **ptr);
    static const char *SimpleQuery(const char *name,const char *closure);
    static ResValue Query(const char *name,const char *closure);
+   static bool QueryBool(const char *name,const char *closure);
 
    enum CmpRes {
       EXACT_PREFIX=0x00,SUBSTR_PREFIX=0x01,
@@ -85,11 +87,11 @@ public:
    static const char *FindVar(const char *name,ResDecl **type);
    static const char *Set(const char *name,const char *closure,const char *value);
 
-   static void Print(FILE *);
    static char *Format(bool with_defaults,bool only_defaults);
    static char **Generator(void);
 
    static const char *BoolValidate(char **value);
+   static const char *TriBoolValidate(char **value);
    static const char *NumberValidate(char **value);
    static const char *UNumberValidate(char **value);
    static const char *FloatValidate(char **value);
@@ -122,6 +124,7 @@ public:
    ~ResDecl();
 
    ResValue Query(const char *closure);
+   bool QueryBool(const char *closure);
 };
 
 class ResValue
@@ -132,7 +135,7 @@ public:
       {
 	 s=s_new;
       }
-   operator bool()
+   bool to_bool()
       {
 	 return ResMgr::str2bool(s);
       }
@@ -155,25 +158,27 @@ public:
    bool is_nil() { return s==0; }
 };
 
-class TimeInterval
+inline bool ResDecl::QueryBool(const char *closure)
+{
+   return Query(closure).to_bool();
+}
+inline bool ResMgr::QueryBool(const char *name,const char *closure)
+{
+   return Query(name,closure).to_bool();
+}
+
+class TimeInterval : public TimeDiff
 {
-   time_t interval;
    bool infty;
    const char *error_text;
 public:
    TimeInterval(const char *);
-   TimeInterval(time_t i)
-      {
-	 interval=i;
-	 infty=false;
-	 error_text=0;
-      }
-   ~TimeInterval();
+   TimeInterval(time_t i) : TimeDiff(i,0) { infty=false; error_text=0; }
+   TimeInterval(const TimeDiff &d) : TimeDiff(d) { infty=false; error_text=0; }
    bool Error() const { return error_text!=0; };
    const char *ErrorText() const { return error_text; }
 
    bool IsInfty() const { return infty; }
-   time_t Seconds() const { return interval; }
 };
 
 class Range
