@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: Resolver.cc,v 1.48 2001/08/28 09:29:23 lav Exp $ */
+/* $Id: Resolver.cc,v 1.50 2001/11/06 13:15:33 lav Exp $ */
 
 #include <config.h>
 
@@ -111,7 +111,7 @@ Resolver::Resolver(const char *h,const char *p,const char *defp,
    addr=0;
    addr_num=0;
    buf=0;
-   use_fork=ResMgr::Query("dns:use-fork",0);
+   use_fork=ResMgr::QueryBool("dns:use-fork",0);
 
    error=0;
 
@@ -244,13 +244,13 @@ int   Resolver::Do()
    {
       if(timeout>0)
       {
-	 if(now-start_time > timeout)
+	 if(now >= start_time+timeout)
 	 {
 	    err_msg=xstrdup(_("host name resolve timeout"));
 	    done=true;
 	    return MOVED;
 	 }
-	 TimeoutS(timeout-(now-start_time));
+	 TimeoutS(timeout-(time_t(now)-start_time));
       }
       return m;
    }
@@ -461,7 +461,7 @@ int SRV_compare(const void *a,const void *b)
 
 void Resolver::LookupSRV_RR()
 {
-   if(!(bool)ResMgr::Query("dns:SRV-query",hostname))
+   if(!ResMgr::QueryBool("dns:SRV-query",hostname))
       return;
 #ifdef HAVE_RES_SEARCH
    const char *tproto=proto?proto:"tcp";
@@ -904,7 +904,7 @@ void ResolverCache::Find(const char *h,const char *p,const char *defp,
    *n=0;
 
    // if cache is disabled for this host, return nothing.
-   if(!(bool)ResMgr::Query("dns:cache-enable",h))
+   if(!ResMgr::QueryBool("dns:cache-enable",h))
       return;
 
    Entry **ptr=FindPtr(h,p,defp,ser,pr);
@@ -926,7 +926,7 @@ void ResolverCache::CacheCheck()
    {
       Entry *s=*scan;
       TimeInterval expire((const char *)ResMgr::Query("dns:cache-expire",s->hostname));
-      if((!expire.IsInfty() && SMTask::now-s->timestamp>expire.Seconds())
+      if((!expire.IsInfty() && SMTask::now>=s->timestamp+expire.Seconds())
       || (count>=countlimit))
       {
 	 *scan=s->next;
