@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: StatusLine.cc,v 1.10 2001/05/29 06:20:28 lav Exp $ */
+/* $Id: StatusLine.cc,v 1.18 2001/11/09 16:00:29 lav Exp $ */
 
 #include <config.h>
 #include <sys/types.h>
@@ -37,8 +37,11 @@
 #include "xmalloc.h"
 
 #include "ResMgr.h"
+#include "misc.h"
 #include "StatusLine.h"
 
+ResDecl res_status_interval ("cmd:status-interval", "0.8s", ResMgr::TimeIntervalValidate,ResMgr::NoClosure);
+
 int  StatusLine::GetWidth()
 {
 #ifdef TIOCGWINSZ
@@ -59,11 +62,11 @@ StatusLine::StatusLine(int new_fd)
 {
    fd=new_fd;
    update_delayed=false;
-   update_time=0;
    strcpy(shown,"");
    strcpy(def_title,"");
    not_term=!isatty(fd);
    LastWidth=GetWidth();
+   Reconfig();
 }
 
 StatusLine::~StatusLine()
@@ -77,7 +80,7 @@ void StatusLine::Clear()
    newstr[0]=0;
    update(newstr);
    update_delayed=false;
-   update_time=0;
+   update_timer.SetMilliSeconds(20);
 
    WriteTitle(def_title, fd);
 }
@@ -103,94 +106,39 @@ void StatusLine::Show(const char *f,...)
    vsnprintf(newstr,sizeof(newstr),f,v);
    va_end(v);
 
-   if(now>update_time)
+   if(update_timer.Stopped())
    {
       update(newstr);
       update_delayed=false;
-      update_time=now;
-   }
-   else if(strcmp(to_be_shown,newstr))
-   {
-      strcpy(to_be_shown,newstr);
-      update_delayed=true;
-      TimeoutS(1);
+      return;
    }
+
+   if(!strcmp(to_be_shown,newstr))
+      return;
+
+   /* not yet */
+   strcpy(to_be_shown,newstr);
+   update_delayed=true;
 }
 
 void StatusLine::WriteTitle(const char *s, int fd) const
 {
-   if(!(bool)ResMgr::Query("cmd:set-term-status", getenv("TERM")))
+   if(!ResMgr::QueryBool("cmd:set-term-status", getenv("TERM")))
       return;
 
-   const char *scan=ResMgr::Query("cmd:term-status", getenv("TERM"));
-   char ch;
-   char str[3];
-   const char *to_add;
+   subst_t subst[] = {
+      { 'a', "\007" },
+      { 'e', "\033" },
+      { 'n', "\n" },
+      { 's', "lftp" },
+      { 'v', VERSION },
 
-   for(;;)
-   {
-      ch=*scan++;
-      if(ch==0)
-	 break;
-
-      if(ch=='\\' && *scan && *scan!='\\')
-      {
-	 ch=*scan++;
-	 switch(ch)
-	 {
-	 case'0':case'1':case'2':case'3':case'4':case'5':case'6':case'7':
-	 {
-	    unsigned len;
-	    unsigned code;
-	    scan--;
-	    sscanf(scan,"%3o%n",&code,&len);
-	    ch=code;
-	    scan+=len;
-	    str[0]=ch;
-	    str[1]=0;
-	    to_add=str;
-	    break;
-	 }
-	 case 'a':
-	    to_add="\007";
-	    break;
-	 case 'e':
-	    to_add="\033";
-	    break;
-	 case 'n':
- 	    to_add="\n";
- 	    break;
-	 case 's':
- 	    to_add="lftp";
- 	    break;
-	 case 'T':
- 	    to_add=s;
- 	    break;
- 	 case 'v':
-	    to_add=VERSION;
-	    break;
-	 default:
-	    str[0]='\\';
-	    str[1]=ch;
-	    str[2]=0;
-	    to_add=str;
-	    break;
-	 }
-      }
-      else
-      {
-	 if(ch=='\\' && *scan=='\\')
-	    scan++;
-	 str[0]=ch;
-	 str[1]=0;
-	 to_add=str;
-      }
-
-      if(to_add==0)
-	 continue;
-
-      write(fd, to_add, strlen(to_add));
-   }
+      { 'T', s },
+      { 0, "" }
+   };
+   char *disp = Subst(ResMgr::Query("cmd:term-status", getenv("TERM")), subst);
+   write(fd, disp, strlen(disp));
+   xfree(disp);
 }
 
 void StatusLine::update(char *newstr)
@@ -235,8 +183,9 @@ void StatusLine::update(char *newstr)
 
    write(fd,"\r",1);
    write(fd,newstr,strlen(newstr));
-}
 
+   update_timer.SetResource("cmd:status-interval",0);
+}
 
 void StatusLine::WriteLine(const char *f,...)
 {
@@ -285,13 +234,11 @@ int StatusLine::Do()
 {
    if(!update_delayed)
       return STALL;
-   if(now>update_time)
+   if(update_timer.Stopped())
    {
       update(to_be_shown);
       update_delayed=false;
-      update_time=now;
       return STALL;
    }
-   TimeoutS(1);
    return STALL;
 }
