@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: commands.cc,v 1.136 2001/09/10 06:13:57 lav Exp $ */
+/* $Id: commands.cc,v 1.151.2.1 2001/11/22 12:30:48 lav Exp $ */
 
 #include <config.h>
 
@@ -46,6 +46,7 @@
 #include "pgetJob.h"
 #include "SleepJob.h"
 #include "FindJob.h"
+#include "FindJobDu.h"
 #include "ChmodJob.h"
 #include "CopyJob.h"
 
@@ -65,6 +66,7 @@
 #include "DummyProto.h"
 #include "QueueFeeder.h"
 #include "lftp_rl.h"
+#include "FileSetOutput.h"
 
 #include "confpaths.h"
 
@@ -75,9 +77,9 @@
 Bookmark lftp_bookmarks;
 History	 cwd_history;
 
-CMD(alias); CMD(anon);   CMD(cd);      CMD(debug);
+CMD(alias); CMD(anon);   CMD(cd);      CMD(debug);	CMD(du);
 CMD(exit);  CMD(get);    CMD(help);    CMD(jobs);
-CMD(kill);  CMD(lcd);    CMD(ls);
+CMD(kill);  CMD(lcd);    CMD(ls);      CMD(cls);
 CMD(open);  CMD(pwd);    CMD(set);
 CMD(shell); CMD(source); CMD(user);    CMD(rm);
 CMD(wait);  CMD(subsh);  CMD(mirror);
@@ -102,7 +104,7 @@ CMD(repeat);CMD(get1);   CMD(history);
 
 const struct CmdExec::cmd_rec CmdExec::static_cmd_table[]=
 {
-   {"!",       cmd_shell,  N_("!<shell_command>"),
+   {"!",       cmd_shell,  N_("!<shell-command>"),
 	 N_("Launch shell or shell command\n")},
    {"(",       cmd_subsh,  N_("(commands)"),
 	 N_("Group commands together to be executed as one command\n"
@@ -146,11 +148,71 @@ const struct CmdExec::cmd_rec CmdExec::static_cmd_table[]=
    {"close",   cmd_close,   "close [-a]",
 	 N_("Close idle connections. By default only with current server.\n"
 	 " -a  close idle connections with all servers\n")},
+   {"cls",     cmd_cls,     N_("[re]cls [opts] [path/][pattern]"),
+	 N_("List remote files. You can redirect output of this command to file\n"
+	    "or via pipe to external command.\n"
+	    "\n"
+	    /* note: I've tried to keep options which are likely to be always
+	     * turned on (via cmd:cls-default, etc) capital, to leave lowercase
+	     * available for options more commonly used manually.  -s/-S is an
+	     * exception; they both seem to be options used manually, so I made
+	     * them align with GNU ls options. */
+	    " -1                   - single-column output\n"
+	    " -B, --basename       - show basename of files only\n"
+	    " -F, --classify       - append indicator (one of /@) to entries\n"
+	    " -l, --long           - use a long listing format\n"
+	    " -q, --quiet          - don't show status\n"
+	    " -s, --size           - print size of each file\n"
+	    "     --filesize       - if printing size, only print size for files\n"
+	    " -i, --nocase         - case-insensitive pattern matching\n"
+	    " -I, --sortnocase     - sort names case-insensitively\n"
+	    " -D, --dirsfirst      - list directories first\n"
+	    "     --sort=OPT       - \"name\", \"size\"\n"
+	    " -S                   - sort by file size\n"
+	    "\n"
+	    "By default, cls output is cached, to see new listing use `recls' or\n"
+	    "`cache flush'.\n"
+	    "\n"
+	    "The variables cls-default and cls-completion-default can be used to\n"
+	    "specify defaults for cls listings and completion listings, respectively.\n"
+	    "For example, to make completion listings show file sizes, set\n"
+	    "cls-completion-default to \"-s\".\n"
+	    "\n"
+	    /* FIXME: poorly worded. another explanation of --filesize: if a person
+	     * wants to only see file sizes for files (not dirs) when he uses -s,
+	     * add --filesize; it won't have any effect unless -s is also used, so
+	     * it can be enabled all the time. (that's also poorly worded, and too
+	     * long.) */
+	    "Tips: Use --filesize and -D to pack the listing better.  If you don't\n"
+	    "always want to see file sizes, --filesize in cls-default will affect the\n"
+	    "-s flag on the commandline as well.  All flags work in\n"
+	    "cls-completion-default.  -i in cls-completion-default makes filename\n"
+	    "completion case-insensitive."
+	    "\n"
+	    "Usage note: directory names must have a trailing slash; \"cls pub/\" to\n"
+	    "be recognized as a directory.\n"
+	   )},
    {"connect", cmd_open,   0,"open"},
    {"command", cmd_command},
    {"debug",   cmd_debug,  N_("debug [<level>|off] [-o <file>]"),
 	 N_("Set debug level to given value or turn debug off completely.\n"
 	 " -o <file>  redirect debug output to the file.\n")},
+   {"du",      cmd_du,  N_("du [options] <dirs>"),
+	 N_("Summarize disk usage.\n"
+	 " -a, --all             write counts for all files, not just directories\n"
+	 " -b, --bytes           print size in bytes\n"
+	 " -c, --total           produce a grand total\n"
+	 " -d, --max-depth=N     print the total for a directory (or file, with --all)\n"
+	 "                       only if it is N or fewer levels below the command\n"
+	 "                       line argument;  --max-depth=0 is the same as\n"
+	 "                       --summarize\n"
+	 " -h, --human-readable  print sizes in human readable format (e.g., 1K 234M 2G)\n"
+	 " -H, --si              likewise, but use powers of 1000 not 1024\n"
+	 " -k, --kilobytes       like --block-size=1024\n"
+	 " -m, --megabytes       like --block-size=1048576\n"
+	 " -S, --separate-dirs   do not include size of subdirectories\n"
+	 " -s, --summarize       display only a total for each argument\n"
+	 "     --exclude=PAT     exclude files that match PAT\n")},
    {"echo",    cmd_echo,   0},
    {"exit",    cmd_exit,   N_("exit [<code>|bg]"),
 	 N_("exit - exit from lftp or move to background if jobs are active\n\n"
@@ -172,8 +234,8 @@ const struct CmdExec::cmd_rec CmdExec::static_cmd_table[]=
 	 " -a  use ascii mode (binary is the default)\n"
 	 " -O <base> specifies base directory or URL where files should be placed\n")},
    {"get1",    cmd_get1,   0,0},
-   {"glob",    cmd_glob,   0,
-	 N_("Usage: glob [OPTS] command args...\n"
+   {"glob",    cmd_glob,   N_("glob [OPTS] <cmd> <args>"),
+	 N_(
 	 "Expand wildcards and run specified command.\n"
 	 "Options can be used to expand wildcards to list of files, directories,\n"
 	 "or both types. Type selection is not very reliable and depends on server.\n"
@@ -214,7 +276,8 @@ const struct CmdExec::cmd_rec CmdExec::static_cmd_table[]=
 	 N_("List remote files. You can redirect output of this command to file\n"
 	 "or via pipe to external command.\n"
 	 "By default, ls output is cached, to see new listing use `rels' or\n"
-	 "`cache flush'.\n")},
+	 "`cache flush'.\n"
+	 "See also `help cls'.\n")},
    {"mget",    cmd_get,	   N_("mget [OPTS] <files>"),
 	 N_("Gets selected files with expanded wildcards\n"
 	 " -c  continue, reget\n"
@@ -273,8 +336,10 @@ const struct CmdExec::cmd_rec CmdExec::static_cmd_table[]=
 	 N_("Removes specified files with wildcard expansion\n")},
    {"mv",      cmd_mv,	    N_("mv <file1> <file2>"),
 	 N_("Rename <file1> to <file2>\n")},
-   {"nlist",   cmd_ls,     N_("nlist [<args>]"),
-	 N_("List remote file names\n")},
+   {"nlist",   cmd_ls,     N_("[re]nlist [<args>]"),
+	 N_("List remote file names.\n"
+	 "By default, nlist output is cached, to see new listing use `renlist' or\n"
+	 "`cache flush'.\n")},
    {"open",    cmd_open,   N_("open [OPTS] <site>"),
 	 N_("Select a server, URL or bookmark\n"
 	 " -e <cmd>            execute the command just after selecting\n"
@@ -298,9 +363,8 @@ const struct CmdExec::cmd_rec CmdExec::static_cmd_table[]=
    {"pwd",     cmd_pwd,    "pwd [-p]",
 	 N_("Print current remote URL.\n"
 	 " -p  show password\n")},
-   {"queue",   cmd_queue,  0,
-	 N_(
-	 "Usage:\n"
+   {"queue",   cmd_queue,  N_("queue [OPTS] [<cmd>]"),
+	 N_("\n"
 	 "       queue [-n num] <command>\n\n"
 	 "Add the command to queue for current site. Each site has its own command\n"
 	 "queue. `-n' adds the command before the given item in the queue. It is\n"
@@ -320,18 +384,24 @@ const struct CmdExec::cmd_rec CmdExec::static_cmd_table[]=
 	 "unknown remote state and thus will cause reconnect. You cannot\n"
 	 "be sure that any change of remote state because of quoted command\n"
 	 "is solid - it can be reset by reconnect at any time.\n")},
-   {"reget",   cmd_get,    N_("reget [OPTS] <rfile> [-o <lfile>]"),
-	 N_("Same as `get -c'\n")},
-   {"rels",    cmd_ls,	    N_("rels [<args>]"),
-	 N_("Same as `ls', but don't look in cache\n")},
-   {"renlist", cmd_ls,	    N_("renlist [<args>]"),
-	 N_("Same as `nlist', but don't look in cache\n")},
-   {"repeat",  cmd_repeat, 0,
-	 N_("Usage: repeat [delay] [command]\n"
-	 "Repeat specified command with a delay between iterations.\n"
+   {"recls",    cmd_cls,   0,
+	 N_("recls [<args>]"
+	 "Same as `cls', but don't look in cache\n")},
+   {"reget",   cmd_get,    0,
+	 N_("Usage: reget [OPTS] <rfile> [-o <lfile>]"
+	 "Same as `get -c'\n")},
+   {"rels",    cmd_ls,	    0,
+	 N_("Usage: rels [<args>]"
+	    "Same as `ls', but don't look in cache\n")},
+   {"renlist", cmd_ls,	    0,
+	 N_("Usage: renlist [<args>]"
+	 "Same as `nlist', but don't look in cache\n")},
+   {"repeat",  cmd_repeat, N_("repeat [delay] [command]"),
+	 N_("Repeat specified command with a delay between iterations.\n"
 	 "Default delay is one second, default command is empty.\n")},
-   {"reput",   cmd_get,    N_("reput <lfile> [-o <rfile>]"),
-	 N_("Same as `put -c'\n")},
+   {"reput",   cmd_get,    0,
+	 N_("Usage: reput <lfile> [-o <rfile>]"
+	 "Same as `put -c'\n")},
    {"rm",      cmd_rm,	    N_("rm [-r] [-f] <files>"),
 	 N_("Remove remote files\n"
 	    " -r  recursive directory removal, be careful\n"
@@ -372,9 +442,9 @@ const struct CmdExec::cmd_rec CmdExec::static_cmd_table[]=
 	 N_("Same as cat, but filter each file through zcat\n")},
    {"zmore",   cmd_cat,    N_("zmore <files>"),
 	 N_("Same as more, but filter each file through zcat\n")},
-   {"bzcat",    cmd_cat,    N_("bzcat <files>"),
+   {"bzcat",    cmd_cat,    0,
 	 N_("Same as cat, but filter each file through bzcat\n")},
-   {"bzmore",   cmd_cat,    N_("bzmore <files>"),
+   {"bzmore",   cmd_cat,    0,
 	 N_("Same as more, but filter each file through bzcat\n")},
 
    {NULL,NULL}
@@ -454,7 +524,7 @@ Job *CmdExec::builtin_lcd()
    SaveCWD();  // this allocates cwd again
 
    if(interactive)
-      eprintf(_("lcd ok, local cwd=%s\n"),cwd);
+      eprintf(_("lcd ok, local cwd=%s\n"),cwd?cwd:"?");
 
    exit_code=0;
    return 0;
@@ -520,7 +590,7 @@ Job *CmdExec::builtin_exit()
    }
    // Note: one job is this CmdExec.
    if(!bg && top_level
-   && !(bool)ResMgr::Query("cmd:move-background",0) && NumberOfJobs()>1)
+   && !ResMgr::QueryBool("cmd:move-background",0) && NumberOfJobs()>1)
    {
       eprintf(_(
 	 "There are running jobs and `cmd:move-background' is not set.\n"
@@ -957,7 +1027,7 @@ Job *CmdExec::builtin_queue()
 	  * We want an optional argument, but don't use getopt ::, since
 	  * that'll disallow the space between arguments, which we want. */
          arg = args->getarg(args->getindex());
-	 
+
 	 CmdExec *queue=GetQueue(false);
 	 if(!queue) {
 	    eprintf(_("%s: No queue is active.\n"), args->a0());
@@ -1096,7 +1166,7 @@ CMD(ls)
 	 args->insarg(1,"SITE");
    }
 
-   char *a=args->Combine(1);
+   char *a=args->Combine(nlist?1:0);
 
    if(!nlist && args->count()==1 && parent->var_ls && parent->var_ls[0])
       args->Append(parent->var_ls);
@@ -1120,7 +1190,7 @@ CMD(ls)
 
    CopyJob *j=new CopyJob(c,a,op);
    if(!output || output->usesfd(1))
-      j->NoStatus();
+      j->NoStatusOnWrite();
    xfree(a);
    output=0;
    if(!nlist)
@@ -1129,6 +1199,140 @@ CMD(ls)
    return j;
 }
 
+/* this seems to belong here more than in FileSetOutput.cc ... */
+const char *FileSetOutput::parse_argv(ArgV *a)
+{
+   static struct option cls_options[] = {
+      {"basename",no_argument,0,'B'},
+      {"classify",no_argument,0,'F'},
+      {"long",no_argument,0,'l'},
+      {"quiet",no_argument,0,'q'},
+      {"size", no_argument,0,'s'},	/* show size */
+      {"filesize", no_argument,0,0},	/* for files only */
+      {"nocase", no_argument,0,'i'},
+      {"sortnocase", no_argument,0,'I'},
+      {"dirsfirst", no_argument,0,'D'},
+
+      {"sort", required_argument,0, 0},
+      {0, no_argument, 0, 'S'}, // sort by size
+      {0,0,0,0}
+   };
+
+   a->rewind();
+   int opt, longopt;
+   while((opt=a->getopt_long(":1BFilqsDIS", cls_options, &longopt))!=EOF)
+   {
+      switch(opt) {
+      case 0:
+	 if(!strcmp(cls_options[longopt].name, "sort")) {
+	    if(!strcasecmp(optarg, "name")) sort = FileSet::BYNAME;
+	    else if(!strcasecmp(optarg, "size")) sort = FileSet::BYSIZE;
+	    else return _("invalid argument for `--sort'");
+	 } else if(!strcmp(cls_options[longopt].name, "filesize")) {
+	    size_filesonly = true;
+	 }
+	 break;
+      case('1'):
+	 single_column = true;
+         break;
+      case('B'):
+	 basenames = true;
+         break;
+      case('l'):
+	 long_list();
+         break;
+      case('i'):
+	 patterns_casefold = true;
+         break;
+      case('F'):
+         classify=true;
+         break;
+      case('q'):
+	 quiet = true;
+         break;
+      case('s'):
+	 mode |= FileSetOutput::SIZE;
+         break;
+      case('D'):
+	 sort_dirs_first = true;
+	 break;
+      case('I'):
+	 sort_casefold = true;
+	 break;
+      case('S'):
+	 sort = FileSet::BYSIZE;
+	 break;
+
+      default:
+	 /* silly getopt won't give us its error instead of printing it, oh well.
+	  * we only need to never print errors for completion and validation; for
+	  * cls itself we could do it, but that'd be a special case ... */
+	 return _("invalid option");
+      }
+   }
+
+   while(a->getindex()>1)
+      a->delarg(1);
+   a->rewind();
+
+   return NULL;
+}
+
+CMD(cls)
+{
+   exit_code=0;
+
+   const char *op=args->a0();
+   bool re=false;
+
+   if(!output)
+      output=new FDStream(1,"<stdout>");
+
+   FileSetOutput fso;
+   fso.config(output);
+
+   if(!strncmp(op,"re",2))
+      re=true;
+
+   if(const char *err = fso.parse_argv(args)) {
+      if(strcmp(err, "ERR"))
+	      eprintf(_("%s: %s.\n"), op, err);
+      eprintf(_("Try `help %s' for more information.\n"),op);
+      return 0;
+   }
+
+   ArgV arg("", ResMgr::Query("cmd:cls-default", 0));
+   fso.parse_argv(&arg);
+
+   char *a=args->Combine(0);
+
+   FileCopyPeer *src_peer=0;
+   src_peer=new FileCopyPeerCLS(Clone(),args,fso);
+   args=0;
+
+   if(re)
+      src_peer->NoCache();
+   src_peer->SetDate(NO_DATE);
+   src_peer->SetSize(NO_SIZE);
+   FileCopyPeer *dst_peer=new FileCopyPeerFDStream(output,FileCopyPeer::PUT);
+
+   FileCopy *c=FileCopy::New(src_peer,dst_peer,false);
+   c->DontCopyDate();
+   c->LineBuffered();
+   c->Ascii();
+
+   CopyJob *j=new CopyJob(c,a,op);
+   if(fso.quiet)
+      j->NoStatus();
+   else if(output->usesfd(1))
+      j->NoStatusOnWrite();
+
+   xfree(a);
+   output=0;
+
+   return j;
+}
+
 CMD(cat)
 {
    const char *op=args->a0();
@@ -1343,8 +1547,12 @@ CMD(rm)
    bool silent=false;
    const char *opts="+rf";
 
+   bool rmdir = false;
    if(!strcmp(args->a0(),"rmdir"))
+   {
+      rmdir = true;
       opts="+f";
+   }
 
    while((opt=args->getopt(opts))!=EOF)
    {
@@ -1358,7 +1566,7 @@ CMD(rm)
 	 break;
       case('?'):
       print_usage:
-	 eprintf(_("Usage: %s [-r] [-f] files...\n"),args->a0());
+	 eprintf(_("Usage: %s %s[-f] files...\n"),args->a0(), rmdir? "":"[-r] ");
 	 return 0;
       }
    }
@@ -1376,9 +1584,9 @@ CMD(rm)
       return j;
    }
 
-   rmJob *j=(strcmp(args->a0(),"rmdir")
-	     ?new rmJob(Clone(),new ArgV(args->a0()))
-	     :new rmdirJob(Clone(),new ArgV(args->a0())));
+   rmJob *j=(rmdir?
+	     new rmdirJob(Clone(),new ArgV(args->a0())):
+	     new rmJob(Clone(),new ArgV(args->a0())));
 
    while(curr)
    {
@@ -2111,7 +2319,7 @@ CMD(bookmark)
       {
 	 const char *value=args->getnext();
 	 int flags=0;
-	 if((bool)res_save_passwords.Query(session->GetHostName()))
+	 if(res_save_passwords.QueryBool(session->GetHostName()))
 	    flags|=session->WITH_PASSWORD;
 	 if(value==0)
 	 {
@@ -2228,16 +2436,140 @@ CMD(find)
       }
    }
 
-   const char *path=args->getcurr();
-   if(!path)
-      path=".";
-   FinderJob_List *j=new class FinderJob_List(Clone(),path,
+   if(!args->getcurr())
+      args->Append(".");
+   FinderJob_List *j=new class FinderJob_List(Clone(),args,
       output?output:new FDStream(1,"<stdout>"));
    j->set_maxdepth(maxdepth);
    output=0;
    return j;
 }
 
+CMD(du)
+{
+   static struct option du_options[]=
+   {
+      {"all",no_argument,0,'a'},
+      /* alias: both GNU-like max-depth and lftp-like maxdepth;
+       * only document one of them. */
+      {"bytes",no_argument,0,'b'},
+      {"maxdepth",required_argument,0,'d'},
+      {"total",no_argument,0,'c'},
+      {"max-depth",required_argument,0,'d'},
+      {"human-readable",no_argument,0,'h'},
+      {"si",no_argument,0,'H'},
+      {"kilobytes",required_argument,0,'k'},
+      {"megabytes",required_argument,0,'m'},
+      {"separate-dirs",no_argument,0,'S'},
+      {"summarize",no_argument,0,'s'},
+      {"exclude",required_argument,0,0},
+      {0,0,0,0}
+   };
+   int maxdepth = -1;
+   bool max_depth_specified = false;
+   int blocksize = 1024;
+   bool separate_dirs = false;
+   bool summarize_only = false;
+   bool print_totals=false;
+   bool all_files=false;
+   const char *exclude=0;
+
+   exit_code=1;
+
+   const char *op=args->a0();
+
+   args->rewind();
+   int opt, longopt;
+   while((opt=args->getopt_long("+abcd:hHkmsS",du_options,&longopt))!=EOF)
+   {
+      switch(opt)
+      {
+      case 'a':
+	 all_files=true;
+	 break;
+      case 'b':
+	 blocksize = 1;
+	 break;
+      case 'c':
+	 print_totals=true;
+	 break;
+      case 'd':
+	 if(!isdigit((unsigned char)*optarg))
+	 {
+	    eprintf(_("%s: %s - not a number\n"),op,optarg);
+	    return 0;
+	 }
+	 maxdepth = atoi(optarg);
+	 max_depth_specified = true;
+	 break;
+      case 'h':
+	 blocksize = -1024;
+	 break;
+      case 'H':
+	 blocksize = -1000;
+	 break;
+      case 'k': /* the default; here for completeness */
+	 blocksize = 1024;
+	 break;
+      case 'm':
+	 blocksize = 1024*1024;
+	 break;
+      case 's':
+	 summarize_only = true;
+	 break;
+      case 'S':
+	 separate_dirs = true;
+	 break;
+      case 0:
+	 if(!strcmp(du_options[longopt].name, "exclude")) {
+	    exclude=optarg;
+	    break;
+	 }
+	 /* fallthrough */
+      case '?':
+	 eprintf(_("Usage: %s [options] <dirs>\n"),op);
+	 return 0;
+      }
+   }
+
+   if (summarize_only && max_depth_specified && maxdepth == 0)
+      eprintf(_("%s: warning: summarizing is the same as using --max-depth=0\n"), op);
+
+   if (summarize_only && max_depth_specified && maxdepth != 0)
+   {
+      eprintf(_("%s: summarizing conflicts with --max-depth=%i\n"), op, maxdepth);
+      return 0;
+   }
+
+   exit_code=0;
+
+   if (summarize_only)
+      maxdepth = 0;
+
+   if(!args->getcurr())
+      args->Append(".");
+   FinderJob_Du *j=new class FinderJob_Du(Clone(),args,
+      output?output:new FDStream(1,"<stdout>"));
+   args=0;
+   j->PrintDepth(maxdepth);
+   j->SetBlockSize(blocksize);
+   if(print_totals)
+      j->PrintTotals();
+   if(all_files)
+      j->AllFiles();
+   if(separate_dirs)
+      j->SeparateDirs();
+   /* if separate_dirs is on, then there's no point in traversing past
+    * max_print_depth at all */
+   if(separate_dirs && maxdepth != -1)
+      j->set_maxdepth(maxdepth);
+   if(exclude)
+      j->SetExclude(exclude);
+
+   output=0;
+   return j;
+}
+
 CMD(command)
 {
    if(args->count()<2)
@@ -2471,14 +2803,14 @@ CMD(history)
    switch(mode) {
    case READ:
       if(int err = lftp_history_read(fn)) {
-	 eprintf(_("%s: %s: %s\n"), args->a0(), fn, strerror(err));
+	 eprintf("%s: %s: %s\n", args->a0(), fn, strerror(err));
 	 exit_code=1;
       }
       break;
 
    case WRITE:
       if(int err = lftp_history_write(fn)) {
-	 eprintf(_("%s: %s: %s\n"), args->a0(), fn, strerror(err));
+	 eprintf("%s: %s: %s\n", args->a0(), fn, strerror(err));
 	 exit_code=1;
       }
       break;
