@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: complete.cc,v 1.42 2001/05/29 06:14:44 lav Exp $ */
+/* $Id: complete.cc,v 1.55 2001/11/09 16:00:30 lav Exp $ */
 
 #include <config.h>
 
@@ -26,6 +26,10 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <time.h>
+#include <unistd.h>
+#include <glob.h>
+#include <sys/time.h>
+#include <assert.h>
 #include "xalloca.h"
 #include "xmalloc.h"
 #include "FileAccess.h"
@@ -38,11 +42,20 @@
 #include "lftp_rl.h"
 #include "url.h"
 #include "ResMgr.h"
+#include "ColumnOutput.h"
+#include "FileSetOutput.h"
+#include "misc.h"
 
 CDECL_BEGIN
+#define USE_VARARGS 1
+#define PREFER_STDARG 1
 #include "readline/readline.h"
 CDECL_END
 
+#ifndef GLOB_PERIOD
+# define GLOB_PERIOD 0
+#endif
+
 static char *bash_dequote_filename (char *text, int quote_char);
 static int lftp_char_is_quoted(char *string,int eindex);
 
@@ -51,6 +64,7 @@ static int cindex; // index in completion array
 static const char *const *array;
 static char **vars=NULL;
 static FileSet *glob_res=NULL;
+static bool inhibit_tilde;
 
 static bool shell_cmd;
 static bool quote_glob;
@@ -87,7 +101,8 @@ char *command_generator(const char *text,int state)
    return(NULL);
 }
 
-static char *remote_generator(const char *text,int state)
+/* this is used for both remote and local gen now; rename */
+static char *file_generator(const char *text,int state)
 {
    const char *name;
 
@@ -103,7 +118,9 @@ static char *remote_generator(const char *text,int state)
       name=(*glob_res)[cindex++]->name;
       if(!name[0])
 	 continue;
-      if(strncmp(name,text,len)==0)
+      /* it's not our job to be case-sensitive; that's done (or not done)
+       * when we create the glob_res */
+      if(strncasecmp(name,text,len)==0)
 	 return(xstrdup(name));
    }
 
@@ -333,6 +350,11 @@ static completion_type cmd_completion_type(const char *cmd,int start)
 
    if(!strcmp(buf,"cat")
    || !strcmp(buf,"ls")
+   || !strcmp(buf,"cls")
+   || !strcmp(buf,"du")
+   || !strcmp(buf,"find")
+   || !strcmp(buf,"recls")
+   || !strcmp(buf,"rels")
    || !strcmp(buf,"more")
    || !strcmp(buf,"mrm")
    || !strcmp(buf,"mv")
@@ -513,6 +535,8 @@ static bool force_remote=false;
    array of matches, or NULL if there aren't any. */
 static char **lftp_completion (const char *text,int start,int end)
 {
+   FileSetOutput fso;
+
    completion_shell->RestoreCWD();
 
    if(start>end)  // workaround for a bug in readline
@@ -524,6 +548,9 @@ static char **lftp_completion (const char *text,int start,int end)
    rl_ignore_some_completions_function=0;
    shell_cmd=false;
    quote_glob=false;
+   inhibit_tilde=false;
+   delete glob_res;
+   glob_res=0;
 
    completion_type type=cmd_completion_type(rl_line_buffer,start);
 
@@ -548,25 +575,97 @@ static char **lftp_completion (const char *text,int start,int end)
    case VARIABLE:
       generator = vars_generator;
       break;
+
+      char *pat;
    case LOCAL:
-   case LOCAL_DIR:
+   case LOCAL_DIR: {
       if(force_remote || (url::is_url(text) && remote_completion))
       {
 	 if(type==LOCAL_DIR)
 	    type=REMOTE_DIR;
 	 goto really_remote;
       }
-      if(type==LOCAL_DIR)
-	 lftp_rl_set_ignore_some_completions_function(ignore_non_dirs);
+
+      ArgV arg("", ResMgr::Query("cmd:cls-completion-default", 0));
+      fso.parse_argv(&arg);
+
+      bool tilde_expanded=false;
+      const char *home=getenv("HOME");
+      int home_len=xstrlen(home);
+      pat=(char*)alloca((len+home_len)*2+10);
+      if(len>0 && home_len>0 && text[0]=='~' && (len==1 || text[1]=='/'))
+      {
+	 glob_quote(pat,home,home_len);
+	 glob_quote(pat+strlen(pat),text+1,len-1);
+	 if(len==1)
+	    strcat(pat,"/");
+	 tilde_expanded=true;
+	 inhibit_tilde=false;
+      }
+      else
+      {
+	 glob_quote(pat,text,len);
+	 inhibit_tilde=true;
+      }
+
+      /* if we want case-insensitive matching, we need to match everything
+       * in the dir and weed it ourselves (let the generator do it), since
+       * glob() has no casefold option */
+      if(fso.patterns_casefold) {
+	 rl_variable_bind("completion-ignore-case", "1");
+
+	 /* strip back to the last / */
+	 char *sl = strrchr(pat, '/');
+	 if(sl) *++sl = 0;
+	 else pat[0] = 0;
+      } else {
+	 rl_variable_bind("completion-ignore-case", "0");
+      }
+
+      strcat(pat,"*");
+
+      glob_t pglob;
+      glob(pat,GLOB_PERIOD,NULL,&pglob);
+      glob_res=new FileSet;
+      for(int i=0; i<(int)pglob.gl_pathc; i++)
+      {
+	 char *src=pglob.gl_pathv[i];
+	 if(tilde_expanded && home_len>0)
+	 {
+	    src+=home_len-1;
+	    *src='~';
+	 }
+	 if(!strcmp(basename_ptr(src), ".")) continue;
+	 if(!strcmp(basename_ptr(src), "..")) continue;
+	 if(type==LOCAL_DIR && not_dir(src)) continue;
+
+	 FileInfo *f = new FileInfo;
+
+	 f->LocalFile(src, false);
+	 glob_res->Add(f);
+      }
+      globfree(&pglob);
+      glob_res->ExcludeDots();
+
+      rl_filename_completion_desired=1;
+      generator = file_generator;
       break;
+   }
    case REMOTE_FILE:
-   case REMOTE_DIR:
+   case REMOTE_DIR: {
       if(!remote_completion && !force_remote)
 	 break; // local
    really_remote:
-      char *pat=(char*)alloca(len*2+10);
+      pat=(char*)alloca(len*2+10);
       glob_quote(pat,text,len);
 
+      if(pat[0]=='~' && pat[1]==0)
+	 strcat(pat,"/");
+
+      if(pat[0]=='~' && pat[1]=='/')
+	 inhibit_tilde=false;
+      else
+	 inhibit_tilde=true;
       strcat(pat,"*");
 
       completion_shell->session->DontSleep();
@@ -574,12 +673,27 @@ static char **lftp_completion (const char *text,int start,int end)
       SignalHook::ResetCount(SIGINT);
       glob_res=NULL;
       rg=new GlobURL(completion_shell->session,pat);
+
+      rl_save_prompt();
+
+      ArgV arg("", ResMgr::Query("cmd:cls-completion-default", 0));
+      fso.parse_argv(&arg);
+
       if(rg)
       {
 	 rg->glob->NoMatchPeriod();
 	 rg->glob->NoInhibitTilde();
+	 if(fso.patterns_casefold) {
+	    rl_variable_bind("completion-ignore-case", "1");
+	    rg->glob->CaseFold();
+	 } else
+	    rl_variable_bind("completion-ignore-case", "0");
 	 if(type==REMOTE_DIR)
 	    rg->glob->DirectoriesOnly();
+
+	 Timer status_timer;
+	 status_timer.SetMilliSeconds(20);
+
 	 for(;;)
 	 {
 	    SMTask::Schedule();
@@ -590,13 +704,36 @@ static char **lftp_completion (const char *text,int start,int end)
 	       SignalHook::ResetCount(SIGINT);
 	       rl_attempted_completion_over = 1;
 	       delete rg;
+
+	       rl_restore_prompt();
+	       rl_clear_message();
+
 	       return 0;
 	    }
+
+	    if(!fso.quiet)
+	    {
+	       /* don't set blank status; if we're operating from cache,
+		* that's all we'll get and it'll look ugly: */
+	       const char *ret = rg->Status();
+	       if(*ret)
+	       {
+		  if(status_timer.Stopped())
+		  {
+		     rl_message ("%s> ", ret);
+		     status_timer.SetResource("cmd:status-interval",0);
+		  }
+	       }
+	    }
+
 	    SMTask::Block();
 	 }
-	 glob_res=rg->GetResult();
+	 glob_res=new FileSet(rg->GetResult());
 	 glob_res->rewind();
       }
+      rl_restore_prompt();
+      rl_clear_message();
+
       if(glob_res->get_fnum()==1)
       {
 	 FileInfo *info=glob_res->curr();
@@ -605,13 +742,12 @@ static char **lftp_completion (const char *text,int start,int end)
 	    rl_completion_append_character='/';
       }
       rl_filename_completion_desired=1;
-      generator = remote_generator;
+      generator = file_generator;
       break;
-
+   }
    } /* end switch */
 
-   if(generator==0)
-      return 0; // default to local
+   assert(generator);
 
    char quoted=((lftp_char_is_quoted(rl_line_buffer,start) &&
 		 strchr(rl_completer_quote_characters,rl_line_buffer[start-1]))
@@ -622,7 +758,6 @@ static char **lftp_completion (const char *text,int start,int end)
 
    char **matches=lftp_rl_completion_matches(textq,generator);
 
-   glob_res=0;
    if(rg)
       delete rg;
 
@@ -645,6 +780,11 @@ leave:
    return matches;
 }
 
+extern "C" void lftp_line_complete()
+{
+   delete glob_res;
+   glob_res=0;
+}
 
 enum { COMPLETE_DQUOTE,COMPLETE_SQUOTE,COMPLETE_BSQUOTE };
 #define completion_quoting_style COMPLETE_BSQUOTE
@@ -733,7 +873,6 @@ backslash_quote (char *string)
     {
       switch (c)
 	{
- 	case '\'':
  	case '(': case ')':
  	case '!': case '{': case '}':		/* reserved words */
  	case '^':
@@ -744,14 +883,14 @@ backslash_quote (char *string)
 	  if(!shell_cmd && !quote_glob)
 	    goto def;
 	case ' ': case '\t': case '\n':		/* IFS white space */
-	case '"': case '\\':		/* quoting chars */
+	case '"': case '\'': case '\\':		/* quoting chars */
 	case '|': case '&': case ';':		/* shell metacharacters */
 	case '<': case '>':
 	  *r++ = '\\';
 	  *r++ = c;
 	  break;
 	case '~':				/* tilde expansion */
-	  if (s == string && glob_res)
+	  if (s == string && inhibit_tilde)
 	    *r++ = '.', *r++ = '/';
 	  goto def;
 	case '#':				/* comment char */
@@ -958,9 +1097,9 @@ bash_quote_filename (char *s, int rtype, char *qcp)
   return ret;
 }
 
-static int skip_double_quoted(char *s, int i)
+static int skip_quoted(const char *s, int i, char q)
 {
-   while(s[i] && s[i]!='"')
+   while(s[i] && s[i]!=q)
    {
       if(s[i]=='\\' && s[i+1])
 	 i++;
@@ -984,9 +1123,10 @@ int lftp_char_is_quoted(char *string,int eindex)
             return 1;
           continue;
         }
-      else if (string[i] == '"')
+      else if (string[i] == '"' || string[i] == '\'')
         {
-          i = skip_double_quoted (string, ++i);
+	  char quote = string[i];
+          i = skip_quoted (string, ++i, quote);
           if (i > eindex)
             return 1;
           i--;  /* the skip functions increment past the closing quote. */
@@ -1046,9 +1186,9 @@ void lftp_readline_init ()
       "lftp",		      // rl_readline_name
       lftp_completion,	      // rl_attempted_completion_function
       lftp_rl_getc,	      // rl_getc_function
-      "\"",		      // rl_completer_quote_characters
-      " \t\n\"",	      // rl_completer_word_break_characters
-      " \t\n\\\">;|&()*?[]~", // rl_filename_quote_characters
+      "\"'",		      // rl_completer_quote_characters
+      " \t\n\"'",	      // rl_completer_word_break_characters
+      " \t\n\\\"'>;|&()*?[]~",// rl_filename_quote_characters
       bash_quote_filename,    // rl_filename_quoting_function
       bash_dequote_filename,  // rl_filename_dequoting_function
       lftp_char_is_quoted);   // rl_char_is_quoted_p
@@ -1056,3 +1196,53 @@ void lftp_readline_init ()
    lftp_rl_add_defun("complete-remote",lftp_complete_remote,-1);
    lftp_rl_bind("Meta-Tab","complete-remote");
 }
+
+extern "C" void completion_display_list (char **matches, int len)
+{
+   FDStream o(1,"<stdout>");
+   Buffer *b = new Buffer;
+
+   if(glob_res) {
+      /* Our last completion action was of files, and we kept that
+       * data around.  Take the files in glob_res which are in matches
+       * and put them in another FileSet.  (This is a little wasteful,
+       * since we're going to use it briefly and discard it, but it's
+       * not worth adding temporary-filtering options to FileSet.) */
+      FileSet tmp;
+      for(int i = 1; i <= len; i++) {
+	 FileInfo *fi = glob_res->FindByName(matches[i]);
+	 assert(fi);
+	 tmp.Add(new FileInfo(*fi));
+      }
+
+      FileSetOutput fso;
+      fso.config(&o);
+
+      ArgV arg("", ResMgr::Query("cmd:cls-completion-default", 0));
+      fso.parse_argv(&arg);
+
+      fso.print(tmp, b);
+   } else {
+      /* Just pass it through ColumnInfo. */
+      ColumnOutput c;
+      for(int i = 1; i <= len; i++) {
+	 c.append();
+	 c.add(matches[i], "");
+      }
+      c.print(b, fd_width(o.getfd()), isatty(o.getfd()));
+   }
+
+   const char *buf;
+   int sz;
+   b->Get(&buf, &sz);
+
+   while(sz) {
+      int ret = write(1, buf, sz);
+      if(ret <= 0)
+	 break; /* oops */
+      sz -= ret;
+      buf += ret;
+   }
+
+   SMTask::Delete(b);
+}
