@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: ftpclass.cc,v 1.203 2001/09/12 07:16:44 lav Exp $ */
+/* $Id: ftpclass.cc,v 1.213.2.2 2001/12/03 16:15:49 lav Exp $ */
 
 #include <config.h>
 
@@ -88,7 +88,6 @@ const bool Ftp::ftps=false;
 #define FTPS_DATA_PORT 989
 
 #define super NetAccess
-#define peer_sa (peer[peer_curr])
 
 #define is5XX(code) ((code)>=500 && (code)<600)
 #define is4XX(code) ((code)>=400 && (code)<500)
@@ -682,7 +681,7 @@ int Ftp::Handle_PASV()
    p[0]=p0; p[1]=p1;
 
    if((a0==0 && a1==0 && a2==0 && a3==0)
-   || ((bool)Query("fix-pasv-address",hostname)
+   || (QueryBool("fix-pasv-address",hostname)
        && InPrivateNetwork(&data_sa) && !InPrivateNetwork(&peer_sa)))
    {
       // broken server, try to fix up
@@ -913,6 +912,8 @@ void Ftp::InitFtp()
 
    memset(&data_sa,0,sizeof(data_sa));
 
+   disconnect_on_close=false;
+
    Reconfig();
 }
 Ftp::Ftp() : super()
@@ -925,12 +926,19 @@ Ftp::Ftp(const Ftp *f) : super(f)
 
    state=INITIAL_STATE;
    flags=f->flags&MODES_MASK;
+   xfree(home_auto);
+   home_auto=xstrdup(f->home_auto);
 }
 
 Ftp::~Ftp()
 {
    Close();
-   quit_sent=true;
+   Disconnect();
+   if(control_sock!=-1)
+   {
+      FlushSendQueue();
+      ReceiveResp();
+   }
    Disconnect();
 
    xfree(anon_user);
@@ -957,6 +965,34 @@ bool Ftp::AbsolutePath(const char *s)
 	  && s[dev_len-1]=='/'));
 }
 
+void Ftp::PropagateHomeAuto()
+{
+   if(!home_auto)
+      return;
+   for(FA *fo=FirstSameSite(); fo!=0; fo=NextSameSite(fo))
+   {
+      Ftp *o=(Ftp*)fo; // we are sure it is Ftp.
+      if(!o->home_auto)
+      {
+	 o->home_auto=xstrdup(home_auto);
+	 o->dos_path=dos_path;
+	 o->vms_path=vms_path;
+	 if(!o->home)
+	    o->set_home(home_auto);
+      }
+   }
+}
+const char *Ftp::FindHomeAuto()
+{
+   for(FA *fo=FirstSameSite(); fo!=0; fo=NextSameSite(fo))
+   {
+      Ftp *o=(Ftp*)fo; // we are sure it is Ftp.
+      if(o->home_auto)
+	 return o->home_auto;
+   }
+   return 0;
+}
+
 void  Ftp::GetBetterConnection(int level,int count)
 {
    if(level==0 && cwd==0)
@@ -981,6 +1017,8 @@ void  Ftp::GetBetterConnection(int level,int count)
 	 {
 	    if((o->flags&NOREST_MODE) && o->real_pos>0x1000)
 	       continue;
+	    if(o->QueryBool("web-mode",o->hostname))
+	       continue;
 	    o->DataAbort();
 	    o->DataClose();
 	    if(o->control_sock==-1)
@@ -988,7 +1026,7 @@ void  Ftp::GetBetterConnection(int level,int count)
 	 }
 	 else
 	 {
-	    if(o->RespQueueSize()>0)
+	    if(o->RespQueueSize()>0 || o->disconnect_on_close)
 	       continue;
 	 }
       }
@@ -997,30 +1035,6 @@ void  Ftp::GetBetterConnection(int level,int count)
 	 takeover_time=now;
       }
 
-      // connected session (o) must have resolved address
-      if(!peer)
-      {
-	 // copy resolved address so that it would be possible to create
-	 // data connection.
-	 xfree(peer);
-	 peer=(sockaddr_u*)xmemdup(o->peer,o->peer_num*sizeof(*o->peer));
-	 peer_num=o->peer_num;
-	 peer_curr=o->peer_curr;
-      }
-
-      if(home_auto && !o->home_auto)
-      {
-	 o->home_auto=xstrdup(home_auto);
-	 o->dos_path=dos_path;
-	 o->vms_path=vms_path;
-      }
-      else if(!home_auto && o->home_auto)
-      {
-	 home_auto=xstrdup(o->home_auto);
-	 dos_path=o->dos_path;
-	 vms_path=o->vms_path;
-      }
-
       if(level==0 && xstrcmp(real_cwd,o->real_cwd))
 	 continue;
 
@@ -1051,7 +1065,7 @@ int   Ftp::Do()
    // check if idle time exceeded
    if(mode==CLOSED && control_sock!=-1 && idle>0)
    {
-      if(now-idle_start>=idle)
+      if(now >= idle_start+idle)
       {
 	 DebugPrint("---- ",_("Closing idle connection"),1);
 	 Disconnect();
@@ -1059,7 +1073,7 @@ int   Ftp::Do()
 	    idle_start=now+timeout;
 	 return m;
       }
-      TimeoutS(idle_start+idle-now);
+      TimeoutS(idle_start+idle-time_t(now));
    }
 
    if(Error() || eof || quit_sent || mode==CLOSED)
@@ -1115,11 +1129,13 @@ int   Ftp::Do()
       if(!NextTry())
 	 return MOVED;
 
+      peer_sa=peer[peer_curr];
       control_sock=socket(peer_sa.sa.sa_family,SOCK_STREAM,IPPROTO_TCP);
       if(control_sock==-1)
       {
 	 if(peer_curr+1<peer_num)
 	 {
+	    try_time=0;
 	    peer_curr++;
 	    retries--;
 	    return MOVED;
@@ -1184,6 +1200,9 @@ int   Ftp::Do()
       if(!(res&POLLOUT))
 	 goto usual_return;
 
+      addr_len=sizeof(peer_sa);
+      getsockname(control_sock,&peer_sa.sa,&addr_len);
+
 #ifdef USE_SSL
       if(proxy?!strncmp(proxy,"ftps://",7):ftps)
       {
@@ -1200,9 +1219,9 @@ int   Ftp::Do()
 
 #ifdef USE_SSL
       // ssl for anonymous does not make sense.
-      if(!ftps && (bool)Query("ssl-allow") && user && pass)
+      if(!ftps && QueryBool("ssl-allow") && user && pass)
       {
-	 const char *auth=Query("ssl-auth");
+	 const char *auth=Query("ssl-auth",hostname);
 	 SendCmd2("AUTH",auth);
 	 AddResp(234,CHECK_AUTH_TLS);
 	 auth_tls_sent=true;
@@ -1228,7 +1247,7 @@ int   Ftp::Do()
       {
 	 SendCmd("PBSZ 0");
 	 AddResp(0,CHECK_IGNORE);
-	 const char *want_prot=(bool)Query("ssl-protect-data",hostname)?"P":"C";
+	 const char *want_prot=QueryBool("ssl-protect-data",hostname)?"P":"C";
 	 if(*want_prot!=prot)
 	 {
 	    SendCmd2("PROT",want_prot);
@@ -1326,9 +1345,6 @@ int   Ftp::Do()
       }
       takeover_time=NO_DATE;
 
-      assert(peer!=0);
-      assert(peer_curr<peer_num);
-
       if(home==0 && !RespQueueIsEmpty())
 	 goto usual_return;
 
@@ -1399,7 +1415,7 @@ int   Ftp::Do()
 
 	 if(flags&PASSIVE_MODE)
 	 {
-	    if((bool)Query("bind-data-socket")
+	    if(QueryBool("bind-data-socket")
 	    && !IsLoopback(&peer_sa))
 	    {
 	       // connect should come from the same address, else server can refuse.
@@ -1494,7 +1510,7 @@ int   Ftp::Do()
          break;
       case(STORE):
          type=FTP_TYPE_I;
-	 if(!(bool)Query("rest-stor",hostname))
+	 if(!QueryBool("rest-stor",hostname))
 	    real_pos=0;	// some old servers don't handle REST/STOR properly.
          command="STOR";
 	 append_file=true;
@@ -1533,10 +1549,7 @@ int   Ftp::Do()
 	    char *path_to_use=file;
 	    if(!vms_path && !AbsolutePath(file) && real_cwd
 	    && !strncmp(file,real_cwd,len) && file[len]=='/')
-	    {
-	       path_to_use=string_alloca(strlen(file+len)+2);
-	       sprintf(path_to_use,".%s",file+len);
-	    }
+	       path_to_use=file+len+1;
 	    SendCmd2("CWD",path_to_use);
 	    AddResp(RESP_CWD_RMD_DELE_OK,CHECK_CWD);
 	    SetRespPath(file);
@@ -1874,7 +1887,7 @@ int   Ftp::Do()
 	 // but the data can be still unsent in server side kernel buffer.
 	 // So the ftp server can decide the connection is idle for too long
 	 // time and disconnect. This hack is to prevent the above.
-	 if(now-nop_time>=nop_interval)
+	 if(now >= nop_time+nop_interval)
 	 {
 	    // prevent infinite NOOP's
 	    if(nop_offset==pos && nop_count*nop_interval>=timeout)
@@ -1893,7 +1906,7 @@ int   Ftp::Do()
 	       nop_count=0;
 	    nop_offset=pos;
 	 }
-	 TimeoutS(nop_interval-(now-nop_time));
+	 TimeoutS(nop_interval-(time_t(now)-nop_time));
       }
 
       oldstate=state;
@@ -2030,7 +2043,7 @@ system_error:
 
 void Ftp::SendSiteIdle()
 {
-   if(!(bool)Query("use-site-idle"))
+   if(!QueryBool("use-site-idle"))
       return;
    SendCmd2("SITE IDLE",idle);
    AddResp(0,CHECK_IGNORE);
@@ -2125,9 +2138,16 @@ int Ftp::ReplyLogPriority(int code)
       return 3;
    if(code==250 && mode==CHANGE_DIR)
       return 3;
+   if(code==451 && mode==CLOSED)
+      return 4;
    if(code==550 && mode==ARRAY_INFO
    && !RespQueueIsEmpty() && RespQueue[RQ_head].check_case==CHECK_MDTM)
       return 4;
+   if(code==550 && mode==LIST && line && strstr(line,"No files found"))
+      return 4;
+   if(code==550 && mode==CHANGE_DIR)
+      return 4;
+
    // Error messages
    // 221 is the reply to QUIT, but we don't expect it.
    if(code>=400 || (code==221 && RespQueue[RQ_head].expect!=221))
@@ -2362,7 +2382,8 @@ void  Ftp::DataAbort()
 
    CloseRespQueue();
 
-   if(!(bool)Query("use-abor",hostname) || control_ssl)
+   if(!QueryBool("use-abor",hostname) || control_ssl
+   || RespQueueSize()>1)
    {
       if(copy_mode==COPY_NONE
       && !((flags&PASSIVE_MODE) && addr_received<2))
@@ -2390,7 +2411,7 @@ void  Ftp::DataAbort()
    aborted_data_sock=data_sock;
    data_sock=-1;
 
-   if((bool)Query("web-mode"))
+   if(QueryBool("web-mode"))
       Disconnect();
 }
 
@@ -2438,7 +2459,7 @@ void  Ftp::Disconnect()
    DataAbort();
    DataClose();
    if(control_sock>=0 && state!=CONNECTING_STATE && !quit_sent
-   && RespQueueSize()<2 && (bool)Query("use-quit",hostname))
+   && RespQueueSize()<2 && QueryBool("use-quit",hostname))
    {
       SendCmd("QUIT");
       AddResp(221);
@@ -2464,6 +2485,7 @@ void  Ftp::Disconnect()
    state=INITIAL_STATE;
 
 out:
+   disconnect_on_close=false;
    Timeout(0);
 
    disconnect_in_progress=false;
@@ -2492,8 +2514,8 @@ void  Ftp::DataClose()
       DebugPrint("---- ",_("Closing data socket"),7);
       close(data_sock);
       data_sock=-1;
-      if((bool)Query("web-mode"))
-	 Disconnect();
+      if(QueryBool("web-mode"))
+	 disconnect_on_close=true;
    }
    nop_time=0;
    nop_offset=0;
@@ -2751,6 +2773,8 @@ void  Ftp::Close()
    copy_failed=false;
    CloseRespQueue();
    super::Close();
+   if(disconnect_on_close)
+      Disconnect();
 }
 
 void Ftp::CloseRespQueue()
@@ -2771,6 +2795,7 @@ void Ftp::CloseRespQueue()
       case(CHECK_CWD_STALE):
       case(CHECK_PASV):
       case(CHECK_EPSV):
+      case(CHECK_TRANSFER_CLOSED):
 #ifdef USE_SSL
       case(CHECK_AUTH_TLS):
       case(CHECK_PROT):
@@ -2794,9 +2819,11 @@ void Ftp::CloseRespQueue()
       case(CHECK_PORT):
       case(CHECK_FILE_ACCESS):
       case(CHECK_RNFR):
-      case(CHECK_TRANSFER):
 	 RespQueue[i].check_case=CHECK_IGNORE;
 	 break;
+      case(CHECK_TRANSFER):
+	 RespQueue[i].check_case=CHECK_TRANSFER_CLOSED;
+	 break;
       }
       if(cc!=CHECK_USER)
 	 RespQueue[i].log_resp=false;
@@ -3165,6 +3192,7 @@ void  Ftp::MoveConnectionHere(Ftp *o)
    assert(aborted_data_sock==-1);
    aborted_data_sock=o->aborted_data_sock;
    o->aborted_data_sock=-1;
+   peer_sa=o->peer_sa;
    if(peer_curr>=peer_num)
       peer_curr=0;
    type=o->type;
@@ -3184,7 +3212,7 @@ void  Ftp::MoveConnectionHere(Ftp *o)
    last_rest=o->last_rest;
 
    if(!home)
-      home=xstrdup(home_auto);
+      set_home(home_auto);
 
    set_real_cwd(o->real_cwd);
    o->set_real_cwd(0);
@@ -3399,12 +3427,12 @@ void Ftp::CheckResp(int act)
       if(is2XX(act))
       {
 	 if(!home_auto)
-	    home_auto=ExtractPWD();   // it allocates space.
-	 if(!home)
 	 {
-	    home=xstrdup(home_auto);
-	    ExpandTildeInCWD();
+	    home_auto=ExtractPWD();   // it allocates space.
+	    PropagateHomeAuto();
 	 }
+	 if(!home)
+	    set_home(home_auto);
 	 break;
       }
       break;
@@ -3435,6 +3463,16 @@ void Ftp::CheckResp(int act)
       TransferCheck(act);
       break;
 
+   case CHECK_TRANSFER_CLOSED:
+      if(strstr(line,"ABOR")
+      && RespQueueSize()>=2 && RespQueue[RQ_head+1].check_case==CHECK_ABOR)
+      {
+	 DebugPrint("**** ","server bug: 426 reply missed",1);
+	 PopResp();
+	 AbortedClose();
+      }
+      break;
+
 #ifdef USE_SSL
    case CHECK_AUTH_TLS:
       if(is2XX(act))
@@ -3444,7 +3482,7 @@ void Ftp::CheckResp(int act)
       }
       else
       {
-	 if((bool)Query("ssl-force"))
+	 if(QueryBool("ssl-force",hostname))
 	    SetError(LOGIN_FAILED,_("ftp:ssl-force is set and server does not support or allow SSL"));
       }
       break;
@@ -3612,12 +3650,7 @@ bool  Ftp::SameLocationAs(FileAccess *fa)
       return false;
    if(SameConnection(o))
    {
-      if(home && !o->home)
-	 o->home=xstrdup(home);
-      else if(!home && o->home)
-	 home=xstrdup(o->home);
-
-      if(home && xstrcmp(home,o->home))
+      if(home && o->home && strcmp(home,o->home))
 	 return false;
 
       if(!cwd || !o->cwd)
@@ -3667,18 +3700,18 @@ int   Ftp::Done()
    abort();
 }
 
-void Ftp::Connect(const char *new_host,const char *new_port)
+void Ftp::ResetLocationData()
 {
-   super::Connect(new_host,new_port);
+   super::ResetLocationData();
    flags=0;
-
    dos_path=false;
    vms_path=false;
    mdtm_supported=true;
    size_supported=true;
    site_chmod_supported=true;
-
-   Reconfig();
+   xfree(home_auto);
+   home_auto=xstrdup(FindHomeAuto());
+   Reconfig(0);
    state=INITIAL_STATE;
 }
 
@@ -3698,18 +3731,18 @@ void Ftp::Reconfig(const char *name)
 
    const char *c=closure;
 
-   SetFlag(SYNC_MODE,	Query("sync-mode",c));
-   SetFlag(PASSIVE_MODE,Query("passive-mode",c));
-   rest_list = Query("rest-list",c);
+   SetFlag(SYNC_MODE,	QueryBool("sync-mode",c));
+   SetFlag(PASSIVE_MODE,QueryBool("passive-mode",c));
+   rest_list = QueryBool("rest-list",c);
 
    nop_interval = Query("nop-interval",c);
 
-   allow_skey = Query("skey-allow",c);
-   force_skey = Query("skey-force",c);
-   verify_data_address = Query("verify-address",c);
-   verify_data_port = Query("verify-port",c);
+   allow_skey = QueryBool("skey-allow",c);
+   force_skey = QueryBool("skey-force",c);
+   verify_data_address = QueryBool("verify-address",c);
+   verify_data_port = QueryBool("verify-port",c);
 
-   use_stat = Query("use-stat",c);
+   use_stat = QueryBool("use-stat",c);
    stat_interval = Query("stat-interval",c);
 
    xfree(list_options);
@@ -3720,12 +3753,11 @@ void Ftp::Reconfig(const char *name)
    xfree(anon_pass);
    anon_pass=xstrdup(Query("anon-pass",c));
 
-   xfree(home);
    const char *h=QueryStringWithUserAtHost("home");
    if(h && h[0] && AbsolutePath(h))
-      home=xstrdup(h);
+      set_home(h);
    else
-      home=xstrdup(home_auto);
+      set_home(home_auto);
 
    if(NoProxy(hostname))
       SetProxy(0);
