@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1996-1999 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2001 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: ftpclass.h,v 1.75 2001/08/22 07:19:07 lav Exp $ */
+/* $Id: ftpclass.h,v 1.80.2.2 2001/12/03 17:58:21 lav Exp $ */
 
 #ifndef FTPCLASS_H
 #define FTPCLASS_H
@@ -93,7 +93,8 @@ class Ftp : public NetAccess
       CHECK_USER_PROXY,	// check response for USER sent to proxy
       CHECK_PASS,	// check response for PASS
       CHECK_PASS_PROXY,	// check response for PASS sent to proxy
-      CHECK_TRANSFER	// generic check for transfer
+      CHECK_TRANSFER,	// generic check for transfer
+      CHECK_TRANSFER_CLOSED // check for transfer complete when Close()d.
 #ifdef USE_SSL
       ,CHECK_AUTH_TLS,
       CHECK_PROT
@@ -139,6 +140,8 @@ class Ftp : public NetAccess
    void	 proxy_LoginCheck(int);
    void	 proxy_NoPassReqCheck(int);
    char *ExtractPWD();
+   void	 PropagateHomeAuto();
+   const char *FindHomeAuto();
    void	 CatchDATE(int);
    void	 CatchDATE_opt(int);
    void	 CatchSIZE(int);
@@ -153,6 +156,7 @@ class Ftp : public NetAccess
    int   control_sock;
    int   data_sock;
    int	 aborted_data_sock;
+   sockaddr_u peer_sa;
    bool	 quit_sent;
    bool	 fixed_pasv; // had to fix PASV address.
 
@@ -253,11 +257,11 @@ private:
 
    int	 addr_received;	// state of PASV
 
-   bool	 wait_flush:1;	// wait until all responces come
-   bool	 ignore_pass:1;	// logged in just with user
-   bool  verify_data_address:1;
-   bool  verify_data_port:1;
-   bool	 rest_list:1;
+   bool	 wait_flush;	// wait until all responces come
+   bool	 ignore_pass;	// logged in just with user
+   bool  verify_data_address;
+   bool  verify_data_port;
+   bool	 rest_list;
    char  *list_options;
 
    void	 GetBetterConnection(int level,int count);
@@ -279,6 +283,8 @@ private:
 
    bool data_address_ok(sockaddr_u *d=0,bool verify_this_data_port=true);
 
+   bool disconnect_on_close;
+
 public:
    enum copy_mode_t { COPY_NONE, COPY_SOURCE, COPY_DEST };
 private:
@@ -315,6 +321,8 @@ public:
    bool	 SameLocationAs(FileAccess *);
    bool	 SameSiteAs(FileAccess *);
 
+   void	 ResetLocationData();
+
    int IsConnected()
    {
       if(control_sock==-1)
@@ -324,8 +332,6 @@ public:
       return 2;
    }
 
-   void	 Connect(const char *h,const char *p);
-
    int   Read(void *buf,int size);
    int   Write(const void *buf,int size);
    int   Buffered();
