@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: lftp.cc,v 1.43 2001/06/18 13:44:54 lav Exp $ */
+/* $Id: lftp.cc,v 1.45.2.1 2001/11/19 10:00:56 lav Exp $ */
 
 #include <config.h>
 
@@ -291,7 +291,8 @@ void  source_if_exist(CmdExec *exec,const char *rc)
 
 static void tty_clear()
 {
-   top_exec->pre_stdout();
+   if(top_exec)
+      top_exec->pre_stdout();
 }
 
 ResDecl res_save_cwd_history
@@ -304,6 +305,7 @@ int   main(int argc,char **argv)
 #endif
 
    setlocale (LC_ALL, "");
+   setlocale (LC_NUMERIC, "C");
    bindtextdomain (PACKAGE, LOCALEDIR);
    textdomain (PACKAGE);
 
@@ -358,7 +360,7 @@ int   main(int argc,char **argv)
    top_exec->AtExit();
    WaitDone(top_exec);
 
-   if((bool)res_save_cwd_history.Query(0))
+   if(res_save_cwd_history.QueryBool(0))
       cwd_history.Save();
 
    if(Job::NumberOfJobs()>0)
@@ -369,6 +371,7 @@ int   main(int argc,char **argv)
    top_exec->KillAll();
    int exit_code=top_exec->ExitCode();
    SMTask::Delete(top_exec);
+   top_exec=0;
    SessionPool::ClearAll();
    LsCache::Flush();
    ProcWait::DeleteAll();
