@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 2000 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 2000-2001 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,7 +21,12 @@
 #ifndef LFTP_SSL_H
 #define LFTP_SSL_H
 
+#ifdef USE_SSL
+# include <openssl/ssl.h>
+
 SSL *lftp_ssl_new(int fd);
 const char *lftp_ssl_strerror(const char *s);
 
+#endif//USE_SSL
+
 #endif//LFTP_SSL_H
