@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1996-2000 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2001 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,9 +18,13 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: misc.cc,v 1.26 2001/07/09 13:21:58 lav Exp $ */
+/* $Id: misc.cc,v 1.42 2001/11/06 13:15:36 lav Exp $ */
 
 #include <config.h>
+#ifdef NEED_TRIO
+#include "trio.h"
+#define vsnprintf trio_vsnprintf
+#endif
 #include "xmalloc.h"
 #include "xstring.h"
 #include <stdio.h>
@@ -30,9 +34,17 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <time.h>
-#ifdef TM_IN_SYS_TIME
+#include <ctype.h>
+#include <sys/ioctl.h>
+
+#ifdef HAVE_TERMIOS_H
+#include <termios.h>
+#endif
+
+#ifdef TIME_WITH_SYS_TIME
 # include <sys/time.h>
 #endif
+
 #include <regex.h>
 #include "misc.h"
 #include "ProcWait.h"
@@ -42,7 +54,7 @@
 const char *dir_file(const char *dir,const char *file)
 {
    if(dir==0 || dir[0]==0)
-      return file;
+      return file?file:dir;
    if(file==0 || file[0]==0)
       return dir;
    if(file[0]=='/')
@@ -256,6 +268,23 @@ void  truncate_file_tree(const char *dir)
    }
 }
 
+int fd_width(int fd)
+{
+   if(fd == -1) return -1;
+   if(!isatty(fd)) return 0;
+
+#ifdef TIOCGWINSZ
+   struct winsize sz;
+   sz.ws_col=sz.ws_row=0;
+   ioctl(fd,TIOCGWINSZ,&sz);
+   if(sz.ws_col==0)
+      sz.ws_col=80;
+   return(sz.ws_col);
+#else /* !TIOCGWINSZ */
+   return 80;
+#endif
+}
+
 char *xgetcwd()
 {
    int size=256;
@@ -337,9 +366,10 @@ int parse_perms(const char *s)
    }
    switch(s[2])
    {
-   case('T'): case('t'): p|=S_ISVTX; break;
-   case('l'): case('L'): p|=S_ISGID; p&=~S_IXGRP; break;
+   case('T'): p|=S_ISVTX; break;
+   case('t'): p|=S_ISVTX; // fall-through
    case('x'): p|=S_IXOTH; break;
+   case('l'): case('L'): p|=S_ISGID; p&=~S_IXGRP; break;
    case('-'): break;
    default: goto bad;
    }
@@ -396,10 +426,9 @@ int parse_year_or_time(const char *year_or_time,int *year,int *hour,int *minute)
    }
    return 0;
 }
-int guess_year(int month,int day,int hour,int minute)
+int guess_year(int month,int day,int hour,int minute,const struct tm *now_p)
 {
-   time_t curr=time(0);
-   struct tm &now=*localtime(&curr);
+   const struct tm &now=*now_p;
    int year=now.tm_year+1900;
    if(((month     *32+        day)*64+       hour)*64+       minute
     > ((now.tm_mon*32+now.tm_mday)*64+now.tm_hour)*64+now.tm_min)
@@ -449,17 +478,95 @@ const char *squeeze_file_name(const char *name,int w)
    Contributed by Roger Beeman <beeman@cisco.com>, with the help of
    Mark Baushke <mdb@cisco.com> and the rest of the Gurus at CISCO.  */
 time_t
-mktime_from_utc (struct tm *t)
+mktime_from_utc (const struct tm *t)
 {
-  time_t tl, tb;
+   struct tm tc;
+   memcpy(&tc, t, sizeof(struct tm));
 
-  tl = mktime (t);
-  if (tl == -1)
-    return -1;
-  tb = mktime (gmtime (&tl));
-  return (tl <= tb ? (tl + (tl - tb)) : (tl - (tb - tl)));
+   /* UTC times are never DST; if we say -1, we'll introduce odd localtime-
+    * dependant errors. */
+
+   tc.tm_isdst = 0;
+
+   time_t tl = mktime (&tc);
+   if (tl == -1)
+      return -1;
+   time_t tb = mktime (gmtime (&tl));
+
+   return (tl <= tb ? (tl + (tl - tb)) : (tl - (tb - tl)));
 }
 
+#ifndef HAVE_UNSETENV
+# ifdef HAVE_ENVIRON
+extern char **environ;
+void unsetenv(const char *env)
+{
+   char **scan=environ;
+   if(!scan)
+      return;
+   int env_len=strlen(env);
+   while(*scan)
+   {
+      if(!strncmp(*scan,env,env_len) && (*scan)[env_len]=='=')
+	 break;
+      scan++;
+   }
+   while(*scan)
+   {
+      *scan=*(scan+1);
+      scan++;
+   }
+}
+# endif
+#endif
+
+static void set_tz(const char *tz)
+{
+   if(!tz)
+   {
+      unsetenv("TZ");
+      return;
+   }
+   static char *put_tz;
+   static int put_tz_alloc;
+   int tz_len=strlen(tz)+4;
+   char *new_tz=put_tz;
+   if(tz_len>put_tz_alloc)
+      new_tz=(char*)xmalloc(put_tz_alloc=tz_len);
+   sprintf(new_tz,"TZ=%s",tz);
+   if(new_tz!=put_tz)
+   {
+      putenv(new_tz);
+      xfree(put_tz);
+      put_tz=new_tz;
+   }
+}
+static char *saved_tz=0;
+static void save_tz()
+{
+   xfree(saved_tz);
+   saved_tz=xstrdup(getenv("TZ"));
+}
+static void restore_tz()
+{
+   set_tz(saved_tz);
+}
+time_t mktime_from_tz(struct tm *t,const char *tz)
+{
+   if(!tz || !*tz)
+      return mktime(t);
+   if(isdigit((unsigned char)*tz) || *tz=='+' || *tz=='-')
+   {
+      char *tz1=string_alloca(strlen(tz)+4);
+      sprintf(tz1,"GMT%s",tz);
+      tz=tz1;
+   }
+   save_tz();
+   set_tz(tz);
+   time_t res=mktime(t);
+   restore_tz();
+   return res;
+}
 
 bool re_match(const char *line,const char *a,int flags)
 {
@@ -472,3 +579,168 @@ bool re_match(const char *line,const char *a,int flags)
    regfree(&re);
    return res;
 }
+
+char *Subst(const char *txt, const subst_t *s)
+{
+   char *buf=0;
+   int buf_size=256;
+
+   if(buf==0)
+      buf=(char*)xmalloc(buf_size);
+
+   char *store=buf;
+
+   char str[3];
+
+   *store=0;
+   while(*txt)
+   {
+      char ch = *txt++;
+      const char *to_add = NULL;
+      if(ch=='\\' && *txt && *txt!='\\')
+      {
+	 ch=*txt++;
+	 if(isdigit(ch) && ch != '8' && ch != '9') {
+	    unsigned len;
+	    unsigned code;
+	    txt--;
+	    sscanf(txt,"%3o%n",&code,&len);
+	    ch=code;
+	    txt+=len;
+	    str[0]=ch;
+	    str[1]=0;
+	    to_add=str;
+	 } else {
+	    for(int i = 0; s[i].from; i++) {
+	       if(s[i].from != ch) continue;
+	       to_add=s[i].to;
+	       if(!to_add) to_add = "";
+	    }
+	    if(!to_add) {
+	       str[0]='\\';
+	       str[1]=ch;
+	       str[2]=0;
+	       to_add=str;
+	    }
+	 }
+      }
+      else
+      {
+	 if(ch=='\\' && *txt=='\\')
+	    txt++;
+	 str[0]=ch;
+	 str[1]=0;
+	 to_add=str;
+      }
+
+      if(to_add==0)
+	 continue;
+
+      int store_index=store-buf;
+      int need=store_index+strlen(to_add)+1;
+      if(buf_size<need)
+      {
+	 while(buf_size<need)
+	    buf_size*=2;
+	 buf=(char*)xrealloc(buf,buf_size);
+	 store=buf+store_index;
+      }
+
+      strcpy(store,to_add);
+      store+=strlen(to_add);
+   }
+   return(buf);
+}
+
+/* if we put escape-handling, etc. in here, the main parser
+ * could possibly use it */
+char **tokenize(const char *str, int *argc)
+{
+   char *buf = xstrdup(str);
+   char **argv = NULL;
+   int _argc;
+   if(!argc) argc = &_argc;
+   *argc = 0;
+
+   for(int i = 0; buf[i]; ) {
+      (*argc)++;
+      argv = (char **) xrealloc(argv, sizeof(char *) * (*argc));
+      argv[*argc-1] = buf+i;
+
+      while(buf[i] && buf[i] != ' ') i++;
+      if(buf[i] == ' ') buf[i++] = 0;
+   }
+
+   argv = (char **) xrealloc(argv, sizeof(char *) * (*argc+1));
+   argv[*argc] = NULL;
+   return argv;
+}
+
+void tokenize_free(char **argv)
+{
+   if(!argv) return;
+   xfree(argv[0]);
+   xfree(argv);
+}
+
+void xgettimeofday(time_t *sec, int *usec)
+{
+#ifdef HAVE_GETTIMEOFDAY
+   struct timeval tv;
+   gettimeofday(&tv,0);
+   if(sec) *sec = tv.tv_sec;
+   if(usec) *usec = tv.tv_usec;
+#else
+   if(sec) time(sec);
+   if(usec) *usec = 0;
+#endif
+}
+
+char *xstrftime(const char *format, const struct tm *tm)
+{
+   char *ret = NULL;
+   int siz = 32;
+
+   struct tm dummy;
+   memset(&dummy, 0, sizeof(dummy));
+   if(tm == NULL)
+      tm = &dummy;
+
+   for(;;)
+   {
+      ret = (char *) xrealloc(ret, siz);
+      int res=strftime(ret, siz, format, tm);
+      if(res>0 && res<siz)
+	 return ret; /* success */
+      /* more space */
+      siz*=2;
+   }
+}
+
+char *xvasprintf(const char *format, va_list ap)
+{
+   char *ret = NULL;
+   int siz = 128;
+
+   for(;;)
+   {
+      ret = (char *) xrealloc(ret, siz);
+      int res=vsnprintf(ret, siz, format, ap);
+      if(res>=0 && res<siz)
+	 return ret;
+      if(res>siz)
+	 siz=res+1;
+      else
+	 siz*=2;
+   }
+}
+
+char *xasprintf(const char *format, ...)
+{
+   char *ret;
+   va_list va;
+   va_start(va, format);
+   ret = xvasprintf(format, va);
+   va_end(va);
+   return ret;
+}
