@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: resource.cc,v 1.61 2001/09/10 06:17:11 lav Exp $ */
+/* $Id: resource.cc,v 1.64 2001/11/12 13:39:01 lav Exp $ */
 
 #include <config.h>
 
@@ -178,6 +178,7 @@ static ResDecl
    ResDecl07 ("ftp:proxy",		  "",	   FtpProxyValidate,0),
    ResDecl08 ("ftp:rest-list",		  "no",	   ResMgr::BoolValidate,0),
    ResDecl09 ("ftp:rest-stor",		  "yes",   ResMgr::BoolValidate,0),
+   ResDecl09a("ftp:timezone",		  "GMT",   0,0),
    ResDecl10 ("ftp:skey-allow",		  "yes",   ResMgr::BoolValidate,0),
    ResDecl11 ("ftp:skey-force",		  "no",    ResMgr::BoolValidate,0),
 #ifdef USE_SSL
@@ -235,7 +236,7 @@ static ResDecl
    ResDecl41 ("net:socket-buffer",	  "0",	   ResMgr::UNumberValidate,0),
    ResDecl42 ("net:socket-maxseg",	  "0",	   ResMgr::UNumberValidate,0),
    ResDecl43 ("net:timeout",		  "300",   ResMgr::UNumberValidate,0),
-   ResDecl44 ("net:connection-limit",	  "0",	   ResMgr::UNumberValidate,ResMgr::NoClosure),
+   ResDecl44 ("net:connection-limit",	  "0",	   ResMgr::UNumberValidate,0),
    ResDecl45 ("net:connection-takeover",  "yes",   ResMgr::BoolValidate,0),
    ResDecl46 ("mirror:time-precision",	  "1s",    ResMgr::TimeIntervalValidate,ResMgr::NoClosure),
    ResDecl47 ("mirror:loose-time-precision","24h", ResMgr::TimeIntervalValidate,ResMgr::NoClosure);
@@ -255,6 +256,8 @@ static ResDecl
    res_query_srv   ("dns:SRV-query",    "no",  ResMgr::BoolValidate,0),
    res_use_fork	   ("dns:use-fork",     "yes", ResMgr::BoolValidate,ResMgr::NoClosure);
 
+static ResDecl
+   fish_shell  ("fish:shell", "/bin/sh",  0,0);
 
 void ResMgr::ClassInit()
 {
