@@ -126,10 +126,8 @@ ParsedURL::ParsedURL(const char *url,bool proto_required,bool use_rfc1738)
    }
 
    // try to extract user name/password
-   scan=base;
-   while(*scan && *scan!='@')
-      scan++;
-   if(*scan=='@')
+   scan=strrchr(base,'@');
+   if(scan)
    {
       *scan++=0;
       user=base;
